#include "cfg.h"

struct_cfg EEMEM cfg_eep; /**< \brief Configuration structure in EEPROM \ingroup group_misc */

// initial configuration setup
void setup_cfg(void) {
    if (read_cfg()) { // wrong configuration
        param_change_period = 10;
        param_brightness_pwm_min = 10;
        param_brightness_pwm_max = 250;
        write_cfg();
    }
}

// update configuration after parameter change
void update_cfg(void) {
    if (cfg_upd_cnt) {
        cfg_upd_cnt--;
        if (cfg_upd_cnt == 0) {
            write_cfg();
        }
    }
}

// read configuration from eeprom
S8 read_cfg(void) {
    U8 result = -1;
    // compare CRC of CFG
    if (crc16_modbus_buf((U8 *)&cfg, sizeof(cfg) - sizeof(cfg.crc), CRC16_MODBUS_INIT) != cfg.crc) {
        // read from EEPROM
        eeprom_read_block(&cfg, &cfg_eep, sizeof(cfg));
        // compare CRC, if differs, fill CFG with defaults
        if (crc16_modbus_buf((U8 *)&cfg, sizeof(cfg) - sizeof(cfg.crc), CRC16_MODBUS_INIT) != cfg.crc) {
            memset(&cfg, 0, sizeof(cfg));
            result = 2;
        } else result = 0;
    } else result = 0;
    return result;
}

// write configuration to eeprom
void write_cfg(void) {
    // calculate CRC
    cfg.crc = crc16_modbus_buf((U8 *)&cfg, sizeof(cfg) - sizeof(cfg.crc), CRC16_MODBUS_INIT);
    // write CFG to EEPROM
    eeprom_write_block(&cfg, &cfg_eep, sizeof(cfg));
}

