#include "kbd.h"

void setup_kbd(void) {
}

void update_kbd(void) {
    kbd_buf[1] = kbd_buf[0] = read_kbd(); // read keyboard state
    
    if (kbd_buf[2] != kbd_buf[1]) { // new key state - restart counter

        kbd_buf[2] = kbd_buf[1];
        kbd_cnt = 0;

    } else {

        if (kbd_cnt < KBD_CNT_MAX) kbd_cnt++; // keypress counter
        
        KBD_BUF_TYPE kbd_co = kbd_buf[2];
        
        // process remote control
        cli(); REMOTE_BUF_TYPE code = remote_code; remote_code = REMOTE_CO_NONE; sei();
        
        if (code == REMOTE_CO_NONE) {
            if (remote_nopress_cnt < 255) remote_nopress_cnt++;
        } else {
            
            if (code == remote_code_prev) {
                if (remote_nopress_cnt < REMOTE_NOPRESS_MIN) {
                    code = REMOTE_CO_NONE;
                }
                remote_nopress_cnt = 0;
            } else {
                remote_nopress_cnt = 0;
                remote_code_prev = code;
                remote_debug = code;
            }

#if DEBUG_REMOTE == 1
            if ((code != REMOTE_CO_NONE)&&(remote_codes_cnt < REMOTE_CODES)) {
                remote_codes[remote_codes_cnt] = code;
                remote_codes_cnt++;
            }
#endif

            if ((code == REMOTE_CO_SONYRM887_OK)||(code == REMOTE_CO_SONYRM887_RIGHT)) {
                if ((cfg.ind_mode == IND_MO_NORMAL)||(cfg.ind_mode == IND_MO_SETUP)||(cfg.ind_mode == IND_MO_SETUPEXT)) {
                    kbd_co = KBD_CO_RIGHT;
                    kbd_cnt = KBD_CNT_SHORT;
                } else if (cfg.ind_mode == IND_MO_NUMBER) {
                    kbd_co = KBD_CO_NUMBER;
                    kbd_cnt = KBD_CNT_SHORT;
                }
            } else if (code == REMOTE_CO_SONYRM887_LEFT) {
                if ((cfg.ind_mode == IND_MO_NORMAL)||(cfg.ind_mode == IND_MO_SETUP)||(cfg.ind_mode == IND_MO_SETUPEXT)) {
                    kbd_co = KBD_CO_LEFT;
                    kbd_cnt = KBD_CNT_SHORT;
                }
            } else if (code == REMOTE_CO_SONYRM887_UP) {
                kbd_co = KBD_CO_UP;
                kbd_cnt = KBD_CNT_SHORT;
            } else if (code == REMOTE_CO_SONYRM887_DOWN) {
                kbd_co = KBD_CO_DOWN;
                kbd_cnt = KBD_CNT_SHORT;
            } else if (code == REMOTE_CO_SONYRM887_MENU) {
                if (cfg.ind_mode == IND_MO_NORMAL) cfg.ind_mode = IND_MO_SETUP;
                kbd_co = KBD_CO_NUMBER;
                kbd_cnt = KBD_CNT_LONG;
            } else if (code == REMOTE_CO_SONYRM887_TV) {
                if (cfg.ind_mode == IND_MO_NORMAL) {
                    kbd_co = KBD_CO_CORR;
                    kbd_cnt = KBD_CNT_SHORT;
                }
            } else if (code == REMOTE_CO_SONYRM887_OSD) {
                if ((cfg.ind_mode == IND_MO_NORMAL)||(cfg.ind_mode == IND_MO_SETUPEXT)) {
                    kbd_co = KBD_CO_SETUPEXT;
                    kbd_cnt = KBD_CNT_LONG;
                }
            }
        }

        // process keypress

        // enter SETUP mode
        if (cfg.ind_mode == IND_MO_NORMAL) {

            if (((kbd_co == KBD_CO_RIGHT)||(kbd_co == KBD_CO_LEFT))&&(kbd_cnt == KBD_CNT_SHORT)) { // enter SETUP mode
                cfg.ind_mode = IND_MO_SETUP;
                ind_exit_tmr = IND_EXIT_SETUP;
                if (kbd_co == KBD_CO_RIGHT) ind_param = 0; else ind_param = IND_SETUP_PARAMS-1;
            }

            else if ((kbd_co == KBD_CO_SETUPEXT)&&(kbd_cnt == KBD_CNT_LONG)) { // enter extended SETUP mode
                cfg.ind_mode = IND_MO_SETUPEXT;
                ind_exit_tmr = IND_EXIT_SETUP;
            }

            else if ((kbd_co == KBD_CO_CORR)&&(kbd_cnt == KBD_CNT_SHORT)) { // do hour correction
                rtc_correct_hour( &rtc ); // correct hour
                rtc_to_ds3231( &rtc ); // update DS3231
                ds3231_wr();
                ind_param = IND_PARAM_HOURMIN;
                ind_cycle_cnt = 0;
            }
        }
        
        // select/edit parameter in SETUP mode
        else if (cfg.ind_mode == IND_MO_SETUP) {

            // enter NUMBER mode
            if ((kbd_co == KBD_CO_NUMBER)&&(kbd_cnt == KBD_CNT_LONG)) {
                cfg.ind_mode = IND_MO_NUMBER;
                ind_number_digit = 0;
                store_cfg();
            }

            // select next parameter
            else if ((kbd_co == KBD_CO_RIGHT)&&(kbd_cnt == KBD_CNT_SHORT)) {
                ind_param++;
                if (ind_param >= IND_SETUP_PARAMS) { // exit from setup
                    cfg.ind_mode = IND_MO_NORMAL;
                    ind_param = 0;
                } else {
                    ind_exit_tmr = IND_EXIT_SETUP;
                }

            }
            // select previous parameter
            else if ((kbd_co == KBD_CO_LEFT)&&(kbd_cnt == KBD_CNT_SHORT)) {
                if (ind_param > 0) ind_param--;
                else {
                    cfg.ind_mode = IND_MO_NORMAL;
                    ind_param = 0;
                }
                ind_exit_tmr = IND_EXIT_SETUP;
            }
            
            // increment parameter
            else if ((kbd_co == KBD_CO_UP)&&((kbd_cnt == KBD_CNT_SHORT)||(kbd_cnt == KBD_CNT_AUTO))) {
                if (kbd_cnt == KBD_CNT_AUTO) kbd_cnt = KBD_CNT_RPT;
                if (ind_param == 0) {
                    rtc.hr++;
                    if (rtc.hr > 23) rtc.hr = 0;
                } else if (ind_param == 1) {
                    rtc.min++;
                    if (rtc.min > 59) rtc.min = 0;
                } else if (ind_param == 2) {
                    rtc.date++;
                    if (rtc.date > 31) rtc.date = 1;
                } else if (ind_param == 3) {
                    rtc.month++;
                    if (rtc.month > 12) rtc.month = 1;
                } else if (ind_param == 4) {
                    if (rtc.year < 2099) rtc.year++;
                }
                // update RTC
                rtc_to_ds3231( &rtc );
                ds3231_wr();
                ind_exit_tmr = IND_EXIT_SETUP;
            }
            
            // decrement parameter
            else if ((kbd_co == KBD_CO_DOWN)&&((kbd_cnt == KBD_CNT_SHORT)||(kbd_cnt == KBD_CNT_AUTO))) {
                if (kbd_cnt == KBD_CNT_AUTO) kbd_cnt = KBD_CNT_RPT;
                if (ind_param == 0) {
                    if (rtc.hr == 0) rtc.hr = 23;
                    else rtc.hr--;
                } else if (ind_param == 1) {
                    if (rtc.min == 0) rtc.min = 59;
                    else rtc.min--;
                } else if (ind_param == 2) {
                    if (rtc.date == 1) rtc.date = 31;
                    else rtc.date--;
                } else if (ind_param == 3) {
                    if (rtc.month == 1) rtc.month = 12;
                    else rtc.month--;
                } else if (ind_param == 4) {
                    if (rtc.year > 2000) rtc.year--;
                }
                // update RTC
                rtc_to_ds3231( &rtc );
                ds3231_wr();
                ind_exit_tmr = IND_EXIT_SETUP;
            }
        }
    
        
        else if (cfg.ind_mode == IND_MO_NUMBER) { // NUMBER mode
        
            if (rtc_present && (kbd_co == KBD_CO_NUMBER) && (kbd_cnt == KBD_CNT_LONG)) { // exit NUMBER mode
                cfg.ind_mode = IND_MO_NORMAL;
                ind_number_digit = 0;
                ind_param = 0;
                store_cfg();
            }

            else if ((kbd_co == KBD_CO_RIGHT)&&(kbd_cnt == KBD_CNT_SHORT)) { // edit next NUMBER digit
                ind_number_digit++;
                if (ind_number_digit > IND_NUMBER_DIGITS) ind_number_digit = 0;
                ind_exit_tmr = IND_EXIT_SETUP;
            }
            else if ((kbd_co == KBD_CO_LEFT)&&(kbd_cnt == KBD_CNT_SHORT)) { // edit previous NUMBER digit
                if (ind_number_digit == 0) ind_number_digit = IND_NUMBER_DIGITS;
                else ind_number_digit--;
                ind_exit_tmr = IND_EXIT_SETUP;
            }

            // increment NUMBER digit
            else if ((kbd_co == KBD_CO_UP)&&((kbd_cnt == KBD_CNT_SHORT)||(kbd_cnt == KBD_CNT_AUTO))&&(ind_number_digit > 0)) {
                if (kbd_cnt == KBD_CNT_AUTO) kbd_cnt = KBD_CNT_RPT;
                cfg.ind_number[ind_number_digit - 1]++;
                if (cfg.ind_number[ind_number_digit - 1] > 10) cfg.ind_number[ind_number_digit - 1] = 0;
                store_cfg();
                ind_exit_tmr = IND_EXIT_SETUP;
            }
            // decrement NUMBER digit
            else if ((kbd_co == KBD_CO_DOWN)&&((kbd_cnt == KBD_CNT_SHORT)||(kbd_cnt == KBD_CNT_AUTO))&&(ind_number_digit > 0)) {
                if (kbd_cnt == KBD_CNT_AUTO) kbd_cnt = KBD_CNT_RPT;
                cfg.ind_number[ind_number_digit - 1]--;
                if (cfg.ind_number[ind_number_digit - 1] > 10) cfg.ind_number[ind_number_digit - 1] = 10;
                store_cfg();
                ind_exit_tmr = IND_EXIT_SETUP;
            }
        }
        
        else if (cfg.ind_mode == IND_MO_SETUPEXT) { // Extended setup mode

            // exit to normal mode
            if ((kbd_co == KBD_CO_SETUPEXT)&&((kbd_cnt == KBD_CNT_SHORT)||(kbd_cnt == KBD_CNT_LONG))) {
                cfg.ind_mode = IND_MO_NORMAL;
            }

            // edit next parameter
            else if ((kbd_co == KBD_CO_RIGHT)&&(kbd_cnt == KBD_CNT_SHORT)) {
                ind_param_ext++;
                if (ind_param_ext >= CFG_PARAMS) ind_param_ext = 0;
                ind_exit_tmr = IND_EXIT_SETUP;
            }
            // edit previous parameter
            else if ((kbd_co == KBD_CO_LEFT)&&(kbd_cnt == KBD_CNT_SHORT)) {
                if (ind_param_ext == 0) ind_param_ext = CFG_PARAMS-1;
                else ind_param_ext--;
                ind_exit_tmr = IND_EXIT_SETUP;
            }

            // increment parameter
            else if ((kbd_co == KBD_CO_UP)&&((kbd_cnt == KBD_CNT_SHORT)||(kbd_cnt == KBD_CNT_AUTO))) {
                if (kbd_cnt == KBD_CNT_AUTO) kbd_cnt = KBD_CNT_RPT;
                cfg.params[ind_param_ext]++;
                store_cfg(); // save config
                ind_exit_tmr = IND_EXIT_SETUP;
            }
            // decrement parameter
            else if ((kbd_co == KBD_CO_DOWN)&&((kbd_cnt == KBD_CNT_SHORT)||(kbd_cnt == KBD_CNT_AUTO))) {
                if (kbd_cnt == KBD_CNT_AUTO) kbd_cnt = KBD_CNT_RPT;
                cfg.params[ind_param_ext]--;
                store_cfg(); // save config
                ind_exit_tmr = IND_EXIT_SETUP;
            }
        }

    }

}

