#include "ul_1wire.h"

#if (defined(__AVR__))
    #include <avr/pgmspace.h>  /*program memory usage for AVR-GCC*/
#endif

U8 ds1w_res_status;

U8 ds1w_bit_rd(U8 data_in) {
    ds1w_delay_bit_before();   /*delay before starting pulse*/
    cli();                     /*disable interrupts*/
    ds1w_dq_wr(0);             /*set DQ to 0 (bit start pulse)*/
    ds1w_delay_bit_start();    /*short delay after DQ level set*/
    ds1w_dq_wr(1);             /*release DQ*/
    ds1w_delay_bit_rd();       /*delay before DQ level read*/
    /*read DQ level to 7th bit of data_in*/
    if (ds1w_dq_rd()) data_in |= 0x80; else data_in &= 0x7f;
    ds1w_delay_bit_after();    /*delay to the rest of bit*/
    sei();                     /*enable interrupts*/
    return(data_in);           /*return result*/
}

void ds1w_bit_wr(U8 data_out) {
    ds1w_delay_bit_before();   /*delay before starting pulse*/
    cli();                     /*disable interrupts*/
    ds1w_dq_wr(0);             /*set DQ to 0 (bit start pulse)*/
    ds1w_delay_bit_start();    /*short delay after DQ level set*/
    ds1w_dq_wr(data_out);      /*set DQ level to bit 0 of abyte*/
    ds1w_delay_bit_after();    /*delay to the rest of bit*/
    ds1w_dq_wr(1);             /*release DQ line*/
    sei();                     /*enable interrupts*/
}

U8 ds1w_byte_rd(void) {
    U8 bits_cnt, byte_out;
    ds1w_delay_byte_before();                /*delay before byte read*/
    byte_out = 0;                            /*clear resulting byte*/
    for (bits_cnt=8; bits_cnt; bits_cnt--) { /*bits read loop*/
        byte_out >>= 1;                      /*shift result right*/
        byte_out = ds1w_bit_rd(byte_out);    /*read next bit*/
    }
    ds1w_delay_byte_after();                 /*delay after byte read*/
    return(byte_out);                        /*return result*/
}

void ds1w_byte_wr(U8 byte_in) {
    U8 bits_cnt;
    ds1w_delay_byte_before();                /*delay before byte write*/
    for (bits_cnt=8; bits_cnt; bits_cnt--) { /*bits write loop*/
        ds1w_bit_wr(byte_in & 1);            /*write lowest bit*/
        byte_in >>= 1;                       /*shift to next bit*/
    }
    ds1w_delay_byte_after();                 /*delay after byte read*/
}

U8 ds1w_res(void) {
    U8 result;
    result = 0;                               /*result of operation*/
    ds1w_delay_reset_before();
    ds1w_dq_wr(1);                            /*release DQ*/
    ds1w_delay_bit_start();
    if (ds1w_dq_rd() == 0) result = 1;        /*if DQ not released (shorted to GND), return result = 2*/
    else {                                    /*else proceed with reset*/
        ds1w_dq_wr(0);                        /*clear DQ*/
        ds1w_delay_bit_start();
        if (ds1w_dq_rd() != 0) {
            result = 2;                       /*if DQ not 0 (shorted to VCC), return result = 2*/
            ds1w_dq_wr(1);                    /*release DQ*/
        } else {                              /*else proceed with reset*/
            ds1w_delay_reset();               /*delay till the rest of reset pulse*/
            ds1w_dq_wr(1);                    /*release DQ*/
            ds1w_delay_presence_before();     /*delay before reading the presense pulse*/
            if (ds1w_dq_rd()) result = 3;     /*if no presence pulse received, return result = 3*/
            else ds1w_delay_presence_after(); /*delay after reading the presense pulse*/
        }
    }
    ds1w_res_status = result;                 /*remember result of reset*/
    return(result);                           /*return result of reset*/
}

/*convert 12-bit temperature data of ds18b20 to signed celsius fixed-point (with tenth)*/
S16 ds1w_12bit2tenth(U16 value) {
    U8 value_sign;
    value_sign = (value >> 8) & 0x80;    /*remember sign of value*/
    if (value_sign) value = 0 - value;   /*convert to absolute value*/
    value = (value >> 1) + (value >> 3); /*convert from 12-bit to fixed point*/
    if (value_sign) value = 0 - value;   /*restore sign of result*/
    return(value);
}

