#include "ul_i2c.h"

// Soft I2C
#if (defined UL_USE_I2C_SOFT) || (defined __DOXYGEN__)
 
    void i2c_soft_start(void) {
        //prepare SDA level in case of repeated start (подготовить уровень на SDA перед стартом)
        i2c_soft_scl_wr_0();
        i2c_soft_sda_wr_1(); i2c_soft_delay_halfbit();
        //1-0 transition on SDA while SCL=1 (перепад на SDA 1-0 при SCL=1)
        i2c_soft_scl_wr_1(); i2c_soft_delay_halfbit();
        i2c_soft_sda_wr_0(); i2c_soft_delay_halfbit();
        i2c_soft_scl_wr_0(); i2c_soft_delay_halfbit();
    }
  
    void i2c_soft_stop(void) {
        //0-1 transition on SDA while SCL=1 (перепад на SDA 0-1 при SCL=1)
        i2c_soft_sda_wr_0(); i2c_soft_delay_halfbit();
        i2c_soft_scl_wr_1(); i2c_soft_delay_halfbit();
        i2c_soft_sda_wr_1(); i2c_soft_delay_halfbit();
    }
  
    void i2c_soft_wr_bit(U8 abit) {
        if (abit) i2c_soft_sda_wr_1(); else i2c_soft_sda_wr_0(); i2c_soft_delay_halfbit();
        i2c_soft_scl_wr_1(); i2c_soft_delay_halfbit();
        i2c_soft_scl_wr_0(); i2c_soft_delay_halfbit();
    }
  
    U8 i2c_soft_rd_bit(void) {
        U8 bit_in;
        i2c_soft_sda_wr_1(); i2c_soft_delay_halfbit();
        i2c_soft_scl_wr_1(); i2c_soft_delay_halfbit();
        bit_in = i2c_soft_sda_rd();
        i2c_soft_scl_wr_0(); i2c_soft_delay_halfbit();
        return(bit_in);
    }
  
    U8 i2c_soft_wr_U8(U8 abyte) {
        for (U8 bits_cnt = 8; bits_cnt; bits_cnt--) {
            i2c_soft_wr_bit(abyte & 0x80);
            abyte <<= 1;
        }
        return(i2c_soft_rd_bit());
    }
  
    U8 i2c_soft_rd_U8(U8 bit_ack) {
        U8 result = 0;
        for (U8 bits_cnt = 8; bits_cnt; bits_cnt--) {
            result <<= 1;
            if (i2c_soft_rd_bit()) result |= 1;
        }
        i2c_soft_wr_bit(bit_ack);
        return(result);
    }

    U8 i2c_soft_chkack(U8 i2c_addr) {
        U8 result;
        i2c_soft_start();    /*START*/
        result = i2c_soft_wr_U8( i2c_addr & 0xfe ); /*send device address in write mode*/
        if (result) result = 1; /*if ACK received,return 0, else 1*/
        i2c_soft_stop(); /*STOP*/
        return(result); /*return 0 if ACK received, else 1*/
    }
#endif


//to include AVR hardware I2C realization define use_i2c_avr (если нужно, подключить функции аппаратного I2C AVR)
#if (defined UL_USE_I2C_AVR) || (defined __DOXYGEN__)
  
    void i2c_avr_wait(void) {
        while (i2c_avr_busy());
    }

    void i2c_avr_cmd(U8 cmd) {
        TWCR = cmd;
    }

    U8 i2c_avr_cmd_wait(U8 cmd) {
        i2c_avr_cmd(cmd);
        i2c_avr_wait();
        return(TWSR);
    }

    U8 i2c_avr_chkack(U8 i2c_addr) {
        U8 result;
        i2c_avr_start();    /*START*/
        TWDR = i2c_addr & 0xfe; /*send device address in write mode*/
        result = i2c_avr_cmd_wait(I2C_AVR_CMD_WR);
        if (result == 0x18) result = 0; else result = 1; /*if ACK received,return 0*/
        i2c_avr_stop(); /*STOP*/
        return(result); /*return 0 if ACK received, else result of operation*/
    }

    U8 i2c_avr_wr_U8(U8 abyte) {
        TWDR = abyte;
        i2c_avr_cmd_wait(I2C_AVR_CMD_WR);
        return(TWSR & 8);
    }

    U8 i2c_avr_rd_U8(U8 ack_bit) {
        i2c_avr_cmd_wait(ack_bit ? I2C_AVR_CMD_RD_NACK : I2C_AVR_CMD_RD_ACK);
        return(TWDR);
    }
#endif



U8 i2c_wr_a8_buf8(U8 addr_i2c, U8 addr_start, U8 *buf, U8 buf_size) {
    i2c_start();
    i2c_wr_U8(addr_i2c);
    i2c_wr_U8(addr_start);
    while (buf_size) {
        i2c_wr_U8(*buf);
        buf++;
        buf_size--;
    }
    i2c_stop();
    return(0);
}

U8 i2c_rd_a8_buf8(U8 addr_i2c, U8 addr_start, U8 *buf, U8 buf_size) {
    /*setup starting address*/
    i2c_start();
    i2c_wr_U8(addr_i2c);
    i2c_wr_U8(addr_start);
    /*if size>0 start reading*/
    if (buf_size) {
        /*repeated start for reading*/
        i2c_start();
        i2c_wr_U8(addr_i2c | 1);
        while (buf_size) {
            /*read next byte to buffer, generate NA in the end*/
            buf_size--;
            *buf = i2c_rd_U8(buf_size == 0);
            buf++;
        }
    }
    /*stop all transactions*/
    i2c_stop();
    return(0);
}


#if defined(UL_USE_I2C_PAGED) || (defined __DOXYGEN__)
    
    //for paged EEPROM read/write define use_i2c_paged (если нужно, подключить функции для записи в устройства с страничной организацией)
    U8  i2c_paged_dev_addr;  /**< \brief I2C paged device address <br>адрес устройства \ingroup group_i2c */
    U16 i2c_paged_size;      /**< \brief I2C paged device size <br>размер \ingroup group_i2c */
    U8  i2c_paged_page_mask; /**< \brief I2C paged device page address mask <br>маска адреса страницы \ingroup group_i2c */
    U16 i2c_paged_byte_addr; /**< \brief I2C paged device byte address <br>текущий адрес записи \ingroup group_i2c */
    U8  i2c_paged_flags;     /**< \brief I2C paged device state <br>текущее состояние записи и другие флаги \ingroup group_i2c */

    #define I2C_PAGED_FLAG_STARTED 1 /**< \brief I2C paged device write started <br>=1 если запись страницы уже начата \ingroup group_i2c */
    #define I2C_PAGED_FLAG_16BIT   2 /**< \brief I2C paged device uses 16-bit address <br>=1 для использования 16-битной адресации \ingroup group_i2c */

    /** \brief start paged write to I2C device <br>начать страничную запись \ingroup group_i2c */
    void i2c_paged_wr_start(U8 dev_addr, U16 byte_addr) {
        i2c_paged_dev_addr  = dev_addr;  //remember device address (запомнить адрес устройства)
        i2c_paged_byte_addr = byte_addr; //remember starting byte (запомнить адрес первого байта)
        i2c_paged_flags &= ~I2C_PAGED_FLAG_STARTED; //clear started flag (обнулить флаг начала записи)
    }

    /** \brief end paged write to I2C device <br>завершить страничную запись \ingroup group_i2c */
    void i2c_paged_wr_stop(void) {
        //if data was written, stop writing and wait to the end of write cycle (если данные уже записывались, прекратить запись и ожидать завершения цикла записи)
        if (i2c_paged_flags & I2C_PAGED_FLAG_STARTED) {
            i2c_stop(); //I2C STOP
            //ACK polling (проверка завершения записи страницы)
            while (i2c_chkack(i2c_paged_dev_addr));
        }
        i2c_stop(); //I2C STOP
        i2c_paged_flags &= ~I2C_PAGED_FLAG_STARTED; //clear STARTED flag (обнулить флаг начатой записи)
    }

    /** \brief paged mode byte write <br>побайтовая запись в страничном режиме \ingroup group_i2c */
    void i2c_paged_wr_U8(U8 adata) {
        //if not overflow, write (если достигнут полный объем, не писать больше)
        if (i2c_paged_byte_addr < i2c_paged_size) {
                //if first data, setup address (если начало записи, дать команду начала записи и настроить адрес)
                if ((i2c_paged_flags & I2C_PAGED_FLAG_STARTED) == 0) {
                    //mark that writing was started (отметить что запись уже происходила)
                    i2c_paged_flags |= I2C_PAGED_FLAG_STARTED;
                    i2c_start();
                    //write device address, write mode (записать адрес устройства)
                    i2c_wr_U8(i2c_paged_dev_addr);
                    //if necessary, write high address byte (в 16-битном режиме адресации записать старший байт адреса)
                    if (i2c_paged_flags & I2C_PAGED_FLAG_16BIT) i2c_wr_U8(i2c_paged_byte_addr >> 8);
                    //write low address byte (записать младший байт адреса)
                    i2c_wr_U8(i2c_paged_byte_addr);
            }
            i2c_wr_U8(adata); //write data byte (записать байт данных)
            i2c_paged_byte_addr++; //increment byte address (увеличить счетчик адреса)
            //if end of current page, end write cycle (если достигнут конец страницы, завершить ее запись)
            if (((U8)i2c_paged_byte_addr & i2c_paged_page_mask) == 0) i2c_paged_wr_stop();
        }
    }

#endif

