#include "ul_i7s.h"

U8 i7s_buf_cnt;
U8 i7s_buf[I7S_BUF_SIZE];
U8 i7s_point;
U8 i7s_minus;
U8 i7s_space;

#if (USE_GCCAVR == 1)
    /** \brief hex symbols array ('0'-'9','A'-'F') in program memory (AVR GCC) <br>массив HEX-символов в памяти (AVR GCC) \ingroup group_i7s */
    U8 PROGMEM i7s_hex_syms[] = {
        (U8)I7S_0 /*0*/, (U8)I7S_1 /*1*/, (U8)I7S_2 /*2*/, (U8)I7S_3 /*3*/,
        (U8)I7S_4 /*4*/, (U8)I7S_5 /*5*/, (U8)I7S_6 /*6*/, (U8)I7S_7 /*7*/,
        (U8)I7S_8 /*8*/, (U8)I7S_9 /*9*/, (U8)I7S_A /*A*/, (U8)I7S_b /*b*/,
        (U8)I7S_C /*C*/, (U8)I7S_d /*d*/, (U8)I7S_E /*E*/, (U8)I7S_F /*F*/
    };
    /** \brief macro for reading HEX symbol from array (AVR GCC)<br>макрос для считывания соответствующего HEX символа (AVR GCC) \ingroup group_i7s */
    U8 i7s_hexdig(U8 adata) { return pgm_read_byte(&i7s_hex_syms[adata]); };
#else
    /** \brief hex symbols array ('0'-'9','A'-'F')<br>массив HEX-символов в памяти \ingroup group_i7s */
    U8 i7s_hex_syms[] = {
        (U8)I7S_0 /*0*/, (U8)I7S_1 /*1*/, (U8)I7S_2 /*2*/, (U8)I7S_3 /*3*/,
        (U8)I7S_4 /*4*/, (U8)I7S_5 /*5*/, (U8)I7S_6 /*6*/, (U8)I7S_7 /*7*/,
        (U8)I7S_8 /*8*/, (U8)I7S_9 /*9*/, (U8)I7S_A /*A*/, (U8)I7S_b /*b*/,
        (U8)I7S_C /*C*/, (U8)I7S_d /*d*/, (U8)I7S_E /*E*/, (U8)I7S_F /*F*/
    };
    /** \brief macro for reading HEX symbol from array<br>макрос для считывания соответствующего HEX символа \ingroup group_i7s */
    U8 i7s_hexdig(U8 adata) { return i7s_hex_syms[adata]; };
#endif



void setup_i7s(void) {
    i7s_point = I7S_POINT;
    i7s_minus = I7S_MINUS;
    i7s_space = I7S_SPACE;
    memset(&i7s_buf, I7S_SPACE, sizeof(i7s_buf));
}
 
void i7s_wr_H8(U8 data, void *buf) {
    *(U8 *)buf = i7s_hexdig(data & 0xf); // put lower digit
    buf--;
    *(U8 *)buf = i7s_hexdig(data >> 4); // put higher digit
}

void i7s_wr_S32(S32 data, void* buf, U8 options) {
    U8 digits_num = (options & 7) + 1; // number of digits
    if (data < 0) { // remember sign of data
        options |= 1;
        data = -data;
    } else options &= ~1;
    if (options & 0x80) // if necessary, convert to BCD
        data = bin2bcd_U32(data, ((digits_num - 1) >> 1) + 1);
    U8 point_pos = ((options >> 4) & 7); // point position
    U8 lz_pos = digits_num - point_pos; // starting leading zero position
    if (point_pos) lz_pos++; // correct if point present
    while (digits_num) { // show next digit loop
        if ((options & 8)&&(data == 0)&&(digits_num < lz_pos)) {
            if (options & 1) {
                *(U8 *)buf = i7s_minus; // show 'minus' sign
                options &= ~1;
            } else *(U8 *)buf = i7s_space; // delete leading zeroes
        } else {
            U8 digit = i7s_hexdig(data & 0xf); // read digit 7-segment code from array
            if ((digits_num == lz_pos) && point_pos)
                digit &= i7s_point; // if necessary, put point mark
            *(U8 *)buf = digit;
        }
        digits_num--;
        data >>= 4;
        buf--;
    }
}

void i7s_wr_trm(int16_t trm, void *buf, U8 options) {
    U8 U8_01, U8_02;
    U8_02 = options & 7;
    U8_01 = trm_err(trm);
    if (U8_01 == TRM_ER_OVER) memset(((U8 *)buf-U8_02), (U8)I7S_SA, (size_t)(U8_02 + 1));
        else if (U8_01 == TRM_ER_UNDER) memset((U8 *)buf-U8_02, (U8)I7S_SD, (size_t)(U8_02 + 1));
            else if (U8_01) memset((U8 *)buf-U8_02, (U8)I7S_SG, (size_t)(U8_02 + 1));
                else i7s_wr_S32(trm, buf, options);
}

