#if !defined(I7S_INCLUDED)
    #define I7S_INCLUDED

#include <string.h>
#include "ul_misc.h"
#include "ul_math.h"
#if defined(USE_GCCAVR)
    #include <avr/pgmspace.h>
#endif


// 7-segment symbols
#define I7S_ALL_ON  (0)
#define I7S_ALL_OFF (0xff)
#define I7S_SPACE   (I7S_ALL_OFF)
#define I7S_MINUS   I7S_SG
#define I7S_POINT   I7S_SH
#define I7S_0    ((U8)(I7S_SA & I7S_SB & I7S_SC & I7S_SD & I7S_SE & I7S_SF))
#define I7S_1    ((U8)(I7S_SB & I7S_SC))
#define I7S_2    ((U8)(I7S_SA & I7S_SB & I7S_SD & I7S_SE & I7S_SG))
#define I7S_3    ((U8)(I7S_SA & I7S_SB & I7S_SC & I7S_SD & I7S_SG))
#define I7S_4    ((U8)(I7S_SB & I7S_SC & I7S_SF & I7S_SG))
#define I7S_5    ((U8)(I7S_SA & I7S_SC & I7S_SD & I7S_SF & I7S_SG))
#define I7S_6    ((U8)(I7S_SA & I7S_SC & I7S_SD & I7S_SE & I7S_SF & I7S_SG))
#define I7S_7    ((U8)(I7S_SA & I7S_SB & I7S_SC))
#define I7S_8    ((U8)(I7S_SA & I7S_SB & I7S_SC & I7S_SD & I7S_SE & I7S_SF & I7S_SG))
#define I7S_9    ((U8)(I7S_SA & I7S_SB & I7S_SC & I7S_SD & I7S_SF & I7S_SG))
#define I7S_A    ((U8)(I7S_SA & I7S_SB & I7S_SC & I7S_SE & I7S_SF & I7S_SG))
#define I7S_b    ((U8)(I7S_SC & I7S_SD & I7S_SE & I7S_SF & I7S_SG))
#define I7S_C    ((U8)(I7S_SA & I7S_SD & I7S_SE & I7S_SF))
#define I7S_Cm   ((U8)(I7S_SA & I7S_SB & I7S_SC & I7S_SD))
#define I7S_c    ((U8)(I7S_SD & I7S_SE & I7S_SG))
#define I7S_d    ((U8)(I7S_SB & I7S_SC & I7S_SD & I7S_SE & I7S_SG))
#define I7S_E    ((U8)(I7S_SA & I7S_SD & I7S_SE & I7S_SF & I7S_SG))
#define I7S_F    ((U8)(I7S_SA & I7S_SE & I7S_SF & I7S_SG))
#define I7S_H    ((U8)(I7S_SB & I7S_SC & I7S_SE & I7S_SF & I7S_SG))
#define I7S_h    ((U8)(I7S_SC & I7S_SE & I7S_SF & I7S_SG))
#define I7S_L    ((U8)(I7S_SD & I7S_SE & I7S_SF))
#define I7S_n    ((U8)(I7S_SC & I7S_SE & I7S_SG))
#define I7S_o    ((U8)(I7S_SC & I7S_SD & I7S_SE & I7S_SG))
#define I7S_P    ((U8)(I7S_SA & I7S_SB & I7S_SE & I7S_SF & I7S_SG))
#define I7S_q    ((U8)(I7S_SA & I7S_SB & I7S_SC & I7S_SF & I7S_SG))
#define I7S_r    ((U8)(I7S_SE & I7S_SG))
#define I7S_t    ((U8)(I7S_SD & I7S_SE & I7S_SF & I7S_SG))
#define I7S_Y    ((U8)(I7S_SB & I7S_SC & I7S_SD & I7S_SF & I7S_SG))
#define I7S_U    ((U8)(I7S_SB & I7S_SC & I7S_SD & I7S_SE & I7S_SF))
#define I7S_u    ((U8)(I7S_SC & I7S_SD & I7S_SE))
#define I7S_Pcyr ((U8)(I7S_SA & I7S_SB & I7S_SC & I7S_SE & I7S_SF))
#define I7S_Gcyr ((U8)(I7S_SA & I7S_SE & I7S_SF))


extern U8 i7s_buf_cnt;
extern U8 i7s_buf[I7S_BUF_SIZE];
extern U8 i7s_point;
extern U8 i7s_minus;
extern U8 i7s_space;


// setup indication parameters
void setup_i7s(void);

// converts 4-bit number to 7-segment digit
U8 i7s_hexdig(U8 adata);

// write 7-segment HEX byte to buf (lower digit), buf-1 (higher digit)
void i7s_wr_H8(U8 data, void *buf);


// 32-bit display flags
#define I7S_NOZEROS 0x08 // supress leding zeros
#define I7S_BIN2BCD 0x80 // convert to BCD

/** \brief write 7-segment 32-bit signed int \b data to \b buf (lowest digit), \b buf-1 (next digit), with \b options :
 <br><b>(options & 7) + 1</b> - number of digits,
 <br><b>((options >> 4) & 7)</b> - point position (counting from right, 0 - without point),
 <br><b>((options & 0x08) == 0x08)</b> - don't show leading zeroes,
 <br><b>((options & 0x80) == 0x80)</b> - previously convert to BCD
 <br>записать 32-bit знаковое \b data в 7-сегментном коде по адресу \b buf (мл.цифра), \b buf-1 (след.цифра) с параметрами \b options :
 <br><b>(options & 7) + 1</b> - количество цифр,
 <br><b>((options >> 4) & 7)</b> - позиция точки (считая справа, 0 - без точки),
 <br><b>((options & 0x08) == 0x08)</b> - не показывать стартовых нулей,
 <br><b>((options & 0x80) == 0x80)</b> - предварительно преобразовать в десятичный вид
 \ingroup group_i7s */
void i7s_wr_S32(S32 data, void* buf, U8 options);


/** \brief write 7-segment 16-bit signed temperature \b data to \b buf (lowest digit), \b buf-1 (next digit), with \b options :
 <br><b>(options & 7) + 1</b> - number of digits,
 <br><b>((options >> 4) & 7)</b> - point position (counting from right, 0 - without point),
 <br><b>((options & 0x08) == 0x08)</b> - don't show leading zeroes,
 <br><b>((options & 0x80) == 0x80)</b> - previously convert to BCD,
 <br>shows upper '-' for overheat (floating sensor) error (trm == trm_er_over),
 <br>shows lower '-' for underheat (shorted sensor) error (trm == trm_er_under),
 <br>else shows center '-' for other errors (trm == trm_er_other),
 <br>записать 32-bit знаковое \b data в 7-сегментном коде по адресу \b buf (мл.цифра), \b buf-1 (след.цифра) с параметрами \b options :
 <br><b>(options & 7) + 1</b> - количество цифр,
 <br><b>((options >> 4) & 7)</b> - позиция точки (считая справа, 0 - без точки),
 <br><b>((options & 0x08) == 0x08)</b> - не показывать стартовых нулей,
 <br><b>((options & 0x80) == 0x80)</b> - предварительно преобразовать в десятичный вид
 <br>выводит верхние '-' для перегрева (или обрыва датчика) (trm == trm_er_over),
 <br>нижние '-' для переохлаждения (или замыкания датчика) (trm == trm_er_under),
 <br>для других ошибок выводит средние '-' (trm == trm_er_other)
 \ingroup group_i7s */
void i7s_wr_trm(S16 trm, void *buf, U8 options);

#endif

