#include "ul_misc.h" /* include misc */
#include "ul_math.h" /* include math */
#include <string.h>  /* include strings/memory blocks */

U8 mul_by5_U8_soft(U8 data_in) {
    return(data_in + (data_in << 2));
}
U8 mul_by5_U16_soft(U16 data_in) {
    return(data_in + (data_in << 2));
}

U8 div_by3_U8_soft(U8 data_in) {
    U16 U16_01, result;
    result = U16_01 = data_in;
    U16_01 <<= 2; result += U16_01;
    U16_01 <<= 2; result += U16_01;
    U16_01 <<= 2; result += U16_01;
    result += 0x55; //correction
    return(result >> 8);
}
U8 div_by3_U8_hmul(U8 data_in) {
    return((((U16)data_in * 0x55) + 0x55) >> 8);
}

U16 div_by3_U16_soft(U16 data_in) {
    U32 U32_01, result;
    result = U32_01 = data_in;
    U32_01 <<= 2; result += U32_01;
    U32_01 <<= 2; result += U32_01;
    U32_01 <<= 2; result += U32_01;
    U32_01 <<= 2; result += U32_01;
    U32_01 <<= 2; result += U32_01;
    U32_01 <<= 2; result += U32_01;
    U32_01 <<= 2; result += U32_01;
    result += 0x5555; //correction
    return(result >> 16);
}
U16 div_by3_U16_hmul(U16 data_in) {
    return((((U32)data_in * 0x5555) + 0x5555) >> 16);
}

U8 div_by5_U8_soft(U8 data_in) {
    U16 U16_01, result;
    result = U16_01 = data_in;
    U16_01 <<= 1; result += U16_01;
    U16_01 <<= 3; result += U16_01;
    U16_01 <<= 1; result += U16_01;
    result += 0x33; //correction
    return(result >> 8);
}
U8 div_by5_U8_hmul(U8 data_in) {
    return((((U16)data_in * 0x33) + 0x33) >> 8);
}

U16 div_by5_U16_soft(U16 data_in) {
    U32 U32_01, result;
    result = U32_01 = data_in;
    U32_01 <<= 1; result += U32_01;
    U32_01 <<= 3; result += U32_01;
    U32_01 <<= 1; result += U32_01;
    U32_01 <<= 3; result += U32_01;
    U32_01 <<= 1; result += U32_01;
    U32_01 <<= 3; result += U32_01;
    U32_01 <<= 1; result += U32_01;
    result += 0x3333; //correction
    return(result >> 16);
}

U16 div_by5_U16_hmul(U16 data_in) {
    return((((U32)data_in * 0x3333) + 0x3333) >> 16);
}


U8 parity_U8(U8 data) {
    inc_parity_U8(data);
    return(data);
}

U8 onebits_cnt_U8(U8 data) {
    U8 bits_cnt;
    for (bits_cnt = 0; data; data >>= 1)
        bits_cnt += data & 1;
    return(bits_cnt);
}

U8 bcd2bin_U8( U8 bcd ) {
    return ( (bcd & 0xf) + (bcd >> 4) * 10 );
}
U8 bin2bcd_U8( U8 bin ) {
    U8 tens = 0;
    while (bin > 9) { bin -= 10; tens++; }
    return ( (tens << 4) + bin );
}

#if (USE_GCCAVR == 1)
    U32 bin2bcd_U32_avr(U32 data, U8 result_bytes) __attribute__((naked)) __attribute__((noinline));
    U32 bin2bcd_U32_avr(U32 data, U8 result_bytes) {
        U32 result;
        asm volatile (
            "push __tmp_reg__ \n"
            "mov __tmp_reg__, %[result_bytes] \n" // __tmp_reg__ = number of bytes of result
            
            "bin2bcd_U32_00: \n"      // correct input number by left shifts
                "mov r31, %A[data] \n"
                "mov %A[data], %B[data] \n"
                "mov %B[data], %C[data] \n"
                "mov %C[data], %D[data] \n"
                "mov %D[data], r31 \n"
                "dec __tmp_reg__ \n"
                "brne bin2bcd_U32_00 \n"

            "eor r26, r26 \n"         // result=0
            "eor r27, r27 \n"
            "eor r30, r30 \n"
            "eor r31, r31 \n"
            "mov __tmp_reg__, %[result_bytes] \n"
            "lsl __tmp_reg__ \n"      // __tmp_reg__ = size in bits
            "lsl __tmp_reg__ \n"
            "lsl __tmp_reg__ \n"
            
            "bin2bcd_U32_01: \n"      // bits shift and correction loop
                "subi r26,-0x33 \n"   // add 0x33
                "sbrs r26, 3 \n"      // if carry to bit 3,
                "subi r26, 3 \n"      // subtract 3
                "sbrs r26, 7 \n"      // if carry to bit 7,
                "subi r26, 0x30 \n"   // subtract 0x30
                "subi r27,-0x33 \n"   // add 0x33
                "sbrs r27, 3 \n"      // if carry to bit 3,
                "subi r27, 3 \n"      // subtract 3
                "sbrs r27, 7 \n"      // if carry to bit 7,
                "subi r27, 0x30 \n"   // subtract 0x30
                "subi r30,-0x33 \n"   // add 0x33
                "sbrs r30, 3 \n"      // if carry to bit 3,
                "subi r30, 3 \n"      // subtract 3
                "sbrs r30, 7 \n"      // if carry to bit 7,
                "subi r30, 0x30 \n"   // subtract 0x30
                "subi r31,-0x33 \n"   // add 0x33
                "sbrs r31, 3 \n"      // if carry to bit 3,
                "subi r31, 3 \n"      // subtract 3
                "sbrs r31, 7 \n"      // if carry to bit 7,
                "subi r31, 0x30 \n"   // subtract 0x30
                "lsl r26 \n"          // shift result number
                "rol r27 \n"
                "rol r30 \n"
                "rol r31 \n"
                
                "sbrc %D[data], 7 \n" // skip if msbit of input == 0
                "ori  r26, 1 \n"      // set lsb of output
                "lsl %A[data] \n"     // shift input number
                "rol %B[data] \n"
                "rol %C[data] \n"
                "rol %D[data] \n"
                
                "dec __tmp_reg__ \n"
                "brne bin2bcd_U32_01 \n" // repeat for all bits
             
            //"mov %A[result], r26 \n"     // move to result
            //"mov %B[result], r27 \n"
            //"mov %C[result], r30 \n"
            //"mov %D[result], r31 \n"

            "mov r22, r26 \n"     // move to result
            "mov r23, r27 \n"
            "mov r24, r30 \n"
            "mov r25, r31 \n"

            "pop __tmp_reg__ \n"
            "ret \n"
            :[result]"=r"(result) // output
            :[data]"r"(data),[result_bytes]"r"(result_bytes) // input
            :"r26","r27","r30","r31" // clobbers
        );
        return result;
    }
#endif

U32 bin2bcd_U32_soft(U32 data, U8 result_bytes) {
    U32 result = 0; /*result*/
    U8 U8_1;
    for (U8_1 = (4 - result_bytes); U8_1; U8_1--) /* adjust input bytes */
        data <<= 8;
    for (U8_1 = (result_bytes << 3); U8_1; U8_1--) { /* bits shift loop */
        U8 U8_2, U8_3;
        /*result BCD nibbles correction*/
        result += 0x33333333;
        /*result correction loop*/
        for (U8_3 = 4; U8_3; U8_3--) {
            U8_2 = result >> 24;
            if (!(U8_2 & 0x08)) U8_2 -= 0x03;
            if (!(U8_2 & 0x80)) U8_2 -= 0x30;
            result <<= 8; /*shift result*/
            result |= U8_2; /*set 8 bits of result*/
        }
        /*shift next bit of input to result*/
        result <<= 1;
        if (((U8)(data >> 24)) & 0x80)
            result |= 1;
        data <<= 1;
    }
    return(result);
}

#if (USE_GCCAVR == 1)
    U32 bcd2bin_U32_avr(U32 data, U8 result_bytes) __attribute__((naked)) __attribute__((noinline));
    U32 bcd2bin_U32_avr(U32 data, U8 result_bytes) {
        U32 result;
        asm volatile (
            "push __tmp_reg__ \n"
            "eor r26, r26 \n"           /* result = 0 */
            "eor r27, r27 \n"
            "eor r30, r30 \n"
            "eor r31, r31 \n"
            "mov __tmp_reg__, %A[result_bytes] \n"   /* __tmp_reg__ = size in bits of input parameter  */
            "lsl __tmp_reg__ \n"
            "lsl __tmp_reg__ \n"
            "lsl __tmp_reg__ \n"

            "bcd2bin_U32_00: \n"        /* bits shift and correction loop */
                "lsr r31 \n"            /* shift out buffer */
                "ror r30 \n"
                "ror r27 \n"
                "ror r26 \n"
             
                "sbrc %A[data], 0 \n"        /* move lowest bit to result */
                "ori  r31, 0x80 \n"
             
                "lsr %D[data] \n"
                "ror %C[data] \n"
                "ror %B[data] \n"
                "ror %A[data] \n"
                
                "sbrc %D[data], 7 \n"        /* if carry to bit 7, */
                "subi %D[data], 0x30 \n"     /* subtract 0x30 */
                "sbrc %D[data], 3 \n"        /* if carry to bit 3, */
                "subi %D[data], 3 \n"        /* subtract 3 */
                "sbrc %C[data], 7 \n"        /* if carry to bit 7, */

                "subi %C[data], 0x30 \n"     /* subtract 0x30 */
                "sbrc %C[data], 3 \n"        /* if carry to bit 3, */
                "subi %C[data], 3 \n"        /* subtract 0x30 */
                "sbrc %B[data], 7 \n"        /* if carry to bit 7, */
                "subi %B[data], 0x30 \n"     /* subtract 0x30 */
                "sbrc %B[data], 3 \n"        /* if carry to bit 3, */
                "subi %B[data], 3 \n"        /* subtract 3 */
                "sbrc %A[data], 7 \n"        /* if carry to bit 7, */
                "subi %A[data], 0x30 \n"     /* subtract 0x30 */
                "sbrc %A[data], 3 \n"        /* if carry to bit 3, */
                "subi %A[data], 3 \n"        /* subtract 3 */
            
                "dec __tmp_reg__ \n"    /* repeat for all bits */
                "brne bcd2bin_U32_00 \n"
             
//            "movw %A[result], r26 \n"          /* adjust result */
//            "movw %C[result], r30 \n"
            "mov r22, r26 \n"     // move to result
            "mov r23, r27 \n"
            "mov r24, r30 \n"
            "mov r25, r31 \n"

            "bcd2bin_U32_01: \n"
                "mov __tmp_reg__,r25 \n"
                "mov r25,r24 \n"
                "mov r24,r23 \n"
                "mov r23,r22 \n"
                "mov r22,__tmp_reg__ \n"
                "dec %[result_bytes] \n"
                "brne bcd2bin_U32_01 \n"

            "pop __tmp_reg__ \n"
            "ret \n"
            :[result]"=r"(result):[data]"r"(data),[result_bytes]"r"(result_bytes):"r0","r26","r27","r30","r31" /*input and output parameters*/
        );
        return result;
    }
#endif

U32 bcd2bin_U32_soft(U32 data, U8 input_bytes) {
    U32 result = 0; /*result*/
    U8 cnt_bits, cnt_bytes, tmp_byte;
    for (cnt_bits = (input_bytes << 3); cnt_bits; cnt_bits--) {
        /*shift next bit*/
        result >>= 1;
        if (((U8)(data)) & 0x01) result |= 0x80000000;
        data >>= 1;
        /* result BCD correction */
        for (cnt_bytes = 4; cnt_bytes; cnt_bytes--) {
            tmp_byte = (data >> 24);
            if (tmp_byte & 0x80) tmp_byte -= 0x30;
            if (tmp_byte & 0x08) tmp_byte -= 0x03;
            data <<= 8;
            data |= tmp_byte;
        }
    }
    /*adjust result bytes*/
    for (cnt_bits = (4 - input_bytes); cnt_bits; cnt_bits--)
        result >>= 8;
    return(result);
}



//ASCII/////////////////////////////////////////////////////////////////////////////////////////////////////////

U8 asc_point = '.';
U8 asc_space = ' ';
U8 asc_minus = '-';

U8 asc2bin_U8(U8 data) {
    if ((data >= '0') && (data <= '9')) data -= '0';
    else if ((data >= 'a') && (data <= 'f')) data -= ('a' - 0xa);
    else if ((data >= 'A') && (data <= 'F')) data -= ('A' - 0xa);
    else data = 0xf0;
    return(data);
}

U8 bin2asc_U8(U8 data) {
    data &= 0xf;
    if (data < 10) data += '0';
    else data += ('A'-0x0a);
    return(data);
}

U32 asc2bin_U32(U8 *buf, S8 size) {
    U32 result;
    U8 flag_negative;
    if (size < 0) {
        flag_negative = 1;
        size = -size;
    } else
        flag_negative = 0;
    result = 0;
    while(size) {
        U8 tmp_U8 = asc2bin_U8(*buf);
        if (tmp_U8 <= 0xf) { /*skip non-HEX symbols*/
            result <<= 4;
            result |= tmp_U8;
        }
        size--;
        if (flag_negative) buf--;
        else buf++; /*go to next symbol*/
    }
    return(result);
}

void bin2asc_U32(U32 data, U8 *buf, S8 size) {
    U8 flag_negative;
    if (size < 0) {
        flag_negative = 1;
        size = -size;
    } else
        flag_negative = 0;
    while (size) {
        *buf = bin2asc_U8(data);
        data >>= 4;
        if (flag_negative) buf--;
        else buf++;
        size--;
        if (data == 0) size = 0; /*stop conversion on zero*/
    }
}

void num2asc_S32(S32 data, U8 *buf, U8 options) {
    U8 cnt_chars, point_pos;
    cnt_chars = (options & 7); /*number of digits*/
    /*convert negative to positive, set '-' flag*/
    if (data < 0) {data = -data; options |= 1;}
    else options &= ~1;
    /*if necessary, convert to BCD*/
    if (options & 0x80) data = bin2bcd_U32(data, (cnt_chars >> 1) + 1);
    cnt_chars++;
    /*calculate point position*/
    point_pos = cnt_chars - ((options >> 4) & 7);
    if (point_pos) point_pos++;
    while (cnt_chars) { /*digits conversion loop*/
        if ((options & 8) && (data == 0) && (cnt_chars < point_pos)) {
            if (options & 1) *buf = asc_minus;
            else *buf = asc_space;
            options &= ~1;
        } else {
            if ((cnt_chars == point_pos) && (options & 0x70)) { /*store point*/
                *buf = asc_point;
                cnt_chars--;
                buf--;
            }
            *buf = bin2asc_U8(data); /*store ASCII digit*/
        }
        cnt_chars--;
        buf--;
        data >>= 4;
    }
    if (options & 1) *buf = asc_minus;
}



//Gray code/////////////////////////////////////////////////////////////////////////////////////////////////////////

U32 bin2gray_U32(U32 data) {
    U32 result = 0;
    U8 tmp_byte, cnt_bits = 0;
    while (data) {
        tmp_byte = data;
        tmp_byte ^= (tmp_byte >> 1);
        result >>= 1;
        if (tmp_byte & 1) result |= 0x80000000;
        data >>= 1;
        cnt_bits++;
    }
    return(result >> (32 - cnt_bits));
}

U8 bin2gray_U8(U8 data) {
    if (((data & 0x01) == 0) != ((data & 0x02) == 0)) data |= 0x01;
    else data &= ~0x01;
    if (((data & 0x02) == 0) != ((data & 0x04) == 0)) data |= 0x02;
    else data &= ~0x02;
    if (((data & 0x04) == 0) != ((data & 0x08) == 0)) data |= 0x04;
    else data &= ~0x04;
    if (((data & 0x08) == 0) != ((data & 0x10) == 0)) data |= 0x08;
    else data &= ~0x08;
    if (((data & 0x10) == 0) != ((data & 0x20) == 0)) data |= 0x10;
    else data &= ~0x10;
    if (((data & 0x20) == 0) != ((data & 0x40) == 0)) data |= 0x20;
    else data &= ~0x20;
    if (((data & 0x40) == 0) != ((data & 0x80) == 0)) data |= 0x40;
    else data &= ~0x40;
    return(data);
}

U8 gray2bin_U8(U8 data) {
    if (((data & 0x40) == 0) != ((data & 0x80) == 0)) data |= 0x40;
    else data &= ~0x40;
    if (((data & 0x20) == 0) != ((data & 0x40) == 0)) data |= 0x20;
    else data &= ~0x20;
    if (((data & 0x10) == 0) != ((data & 0x20) == 0)) data |= 0x10;
    else data &= ~0x10;
    if (((data & 0x08) == 0) != ((data & 0x10) == 0)) data |= 0x08;
    else data &= ~0x08;
    if (((data & 0x04) == 0) != ((data & 0x08) == 0)) data |= 0x04;
    else data &= ~0x04;
    if (((data & 0x02) == 0) != ((data & 0x04) == 0)) data |= 0x02;
    else data &= ~0x02;
    if (((data & 0x01) == 0) != ((data & 0x02) == 0)) data |= 0x01;
    else data &= ~0x01;
    return(data);
}



//CRC////////////////////////////////////////////////////////////////////////////////////////////////////////


/*CRC8 Dallas*/
U8 crc8_ds_upd(U8 data, U8 crc_prev) {
    inc_crc8_ds_upd(data, crc_prev);
    return(crc_prev);
}

U16 crc8_ds_buf(U8 *buf, U16 buf_size, U8 crc_prev) {
    while (buf_size) {
        crc_prev = crc8_ds_upd(*buf, crc_prev);
        buf++;
        buf_size--;
    }
    return(crc_prev);
}



/*CRC16 Dallas*/
U16 crc16_ds_upd(U8 data, U16 crc_prev) {
    inc_crc16_ds_upd(data, crc_prev);
    return(crc_prev);
}

U16 crc16_ds_buf(U8 *buf, U16 buf_size, U16 crc_prev) {
    while (buf_size) {
        crc_prev = crc16_ds_upd(*buf, crc_prev);
        buf++;
        buf_size--;
    }
    return(crc_prev);
}



/*CRC16 Modbus*/
U16 crc16_modbus_upd(U8 data, U16 crc_prev) {
    inc_crc16_modbus_upd(data, crc_prev);
    return(crc_prev);
}

U16 crc16_modbus_buf(U8 *buf, U16 buf_size, U16 crc_prev) {
    while (buf_size) {
        crc_prev = crc16_modbus_upd(*buf, crc_prev);
        buf++;
        buf_size--;
    }
    return(crc_prev);
}



/*CRC16 CCIT (right method)*/
U16 crc16_ccit_upd(U8 data, U16 crc_prev) {
    inc_crc16_ccit_upd(data, crc_prev);
    return(crc_prev);
}

U16 crc16_ccit_augment(U16 crc_prev) {
    inc_crc16_ccit_augment(crc_prev);
    return(crc_prev);
}

U16 crc16_ccit_buf(U8 *buf, U16 buf_size, U16 crc_prev) {
    while (buf_size) {
        crc_prev = crc16_ccit_upd(*buf, crc_prev);
        buf++;
        buf_size--;
    }
    return(crc_prev);
}



/*CRC7 MMC*/
U8 crc7_mmc_upd(U8 data, U8 crc_prev) {
    inc_crc7_mmc_upd(data, crc_prev);
    return(crc_prev & 0x7f);
}

U16 crc7_mmc_buf(U8 *buf, U16 buf_size, U8 crc_prev) {
    while (buf_size) {
        crc_prev = crc7_mmc_upd(*buf, crc_prev);
        buf++;
        buf_size--;
    }
    return(crc_prev);
}

  

/*CRC16 MMC (sort of CRC16 CCIT)*/
U16 crc16_mmc_upd(U8 data, U16 crc_prev) {
    inc_crc16_mmc_upd(data, crc_prev);
    return(crc_prev);
}

U16 crc16_mmc_buf(U8 *buf, U16 buf_size, U16 crc_prev) {
    while (buf_size) {
        crc_prev = crc16_mmc_upd(*buf, crc_prev);
        buf++;
        buf_size--;
    }
    return(crc_prev);
}


U32 hash_rs( U8 *buf, U32 buf_size ) {
    U32 a = 63689;
    U32 hash = HASH_RS_INIT;
    while (buf_size--) {
        hash = (hash * a) + *buf;
        a *= 378551;
        buf++;
    }
    return hash;
}

U32 hash_ly( U8 *buf, U32 buf_size ) {
    U32 hash = HASH_LY_INIT;
    while (buf_size--) {
        hash = (hash * 1664525) + *buf + 1013904223;
        buf++;
    }
    return hash;
}

U8 trm_err(S16 trm) {
    U8 result;
    result = 0;
    if (((U16)trm >> 8) == 0x80) result = ((U8)trm & 0xff) + 1;
    return(result);
}



//buffer operations

U8 bits_rfill_U8(U8 bits_num) {
    U8 byte_out;
    if (bits_num >= 8) byte_out = 0xff;
    else {
        byte_out = 0;
        while (bits_num) {
            byte_out <<= 1;
            byte_out |= 1;
            bits_num--;
        }
    }
    return(byte_out);
}

U8 bits_lfill_U8(U8 bits_num) {
    U8 byte_out;
    if (bits_num >= 8) byte_out = 0xff;
    else {
        byte_out = 0;
        while (bits_num) {
            byte_out >>= 1;
            byte_out |= 0x80;
            bits_num--;
        }
    }
    return(byte_out);
}

U8 bits_mirror_U8(U8 data) {
    U8 result;
    inc_bits_mirror_U8(data, result);
    return(result);
}

U32 bits_mirror_U32(U32 data) {
    inc_bits_mirror_U32(data);
    return(data);
}

U8 bits_nlz_U8(U8 data) {
    U8 cnt_bits;
    cnt_bits = 0;
    while (((data & 0x80) == 0) && (cnt_bits != 8)) {
        data <<= 1;
        cnt_bits++;
    }
    return(cnt_bits);
}

U8 log2ex_U16(U16 data) {
    U16 test_bit = 0x8000;
    U8 result = 15;
    while (test_bit) {
        if (data & test_bit) {
            data &= ~test_bit;
            break;
        }
        test_bit >>= 1;
        result--;
    }
    if ((result > 15) || (data)) result = 0xff;
    return(result);
}

U32 buf_cnt_bytes(U8 *buf, U32 buf_size, U8 abyte) {
    U32 cnt_bytes;
    cnt_bytes = 0;
    while (buf_size) {
        if ((*buf) == abyte) cnt_bytes++;
        buf++;
        buf_size--;
    }
    return(cnt_bytes);
}

void buf_mirror_bytes(U8 *buf, U32 buf_size) {
    if (buf_size > 1) {
        U8 ui8_tmp, *buf1, *buf_track_bits_shift;
        buf1 = buf;
        buf_track_bits_shift = buf + buf_size - 1;
        buf_size >>= 1;
        while (buf_size) {
            ui8_tmp = *(U8 *)buf1;
            *(U8 *)buf1 = *(U8 *)buf_track_bits_shift;
            *(U8 *)buf_track_bits_shift = ui8_tmp;
            buf_size--;
            buf1++;
            buf_track_bits_shift--;
        }
    }
}

void buf_mirror_bytes_bits(U8 *buf, U32 buf_size) {
    while (buf_size) {
        *(U8 *)buf = bits_mirror_U8(*(U8 *)buf);
        buf++;
        buf_size--;
    }
}

void buf_mirror_bits(U8 *buf, U32 buf_size) {
    if (buf_size>0) {
        U8 ui8_tmp, ui8_odd, *buf1, *buf_track_bits_shift;
        ui8_odd = buf_size & 1;
        buf1 = buf;
        buf_track_bits_shift = buf + buf_size - 1;
        buf_size >>= 1;
        while (buf_size) {
            ui8_tmp = bits_mirror_U8(*buf1);
            *buf1 = bits_mirror_U8(*buf_track_bits_shift);
            *buf_track_bits_shift = ui8_tmp;
            buf_size--;
            buf1++;
            buf_track_bits_shift--;
        }
        if (ui8_odd) *(U8 *)buf1 = bits_mirror_U8(*(U8 *)buf1);
    }
}

void buf_shr_bits(U8 *buf, U32 buf_size, U32 bits_num, U8 bit_fill) {
    U8 bit_tmp, bit_prev;
    U32 bytes_cnt;
    //bits shift loop
    while (bits_num--) {
        buf += buf_size; //start from the end of buffer
        bit_prev = bit_fill;
        //1 bit buffer shift loop
        for (bytes_cnt=buf_size; bytes_cnt; bytes_cnt--) {
            buf--;
            bit_tmp = (*buf) & 1;
            (*buf) >>= 1; //shift next byte
            if (bit_prev) (*buf) |= 0x80;
            bit_prev = bit_tmp;
        }
    }
}

void buf_shl_bits(U8 *buf, U32 buf_size, U32 bits_num, U8 bit_fill) {
    U8 bit_tmp, bit_prev;
    U32 bytes_cnt;
    //bits shift loop
    while (bits_num--) {
        bit_prev = bit_fill;
        //1 bit buffer shift loop
        for (bytes_cnt=buf_size; bytes_cnt; bytes_cnt--) {
            bit_tmp = (*buf) & 0x80;
            (*buf) <<= 1;
            if (bit_prev) (*buf) |= 1;
            bit_prev = bit_tmp;
            buf++;
        }
        buf -= buf_size;
    }
}

void buf_shr_bytes(U8 *buf, U32 buf_size, U32 bytes_num, U8 byte_fill) {
    U32 bytes_cnt;
    U8 *buf_ptr;
    while (bytes_num) {
        bytes_cnt = buf_size;
        buf_ptr = buf;
        if (bytes_cnt) {
            while (--bytes_cnt) {
                *buf_ptr = *(buf_ptr + 1);
                buf_ptr++;
            }
            *buf_ptr = byte_fill;
        }
        bytes_num--;
    }
}

void buf_shl_bytes(U8 *buf, U32 buf_size, U32 bytes_num, U8 byte_fill) {
    U32 bytes_cnt;
    U8 *buf_ptr, byte_tmp, byte_prev;
    while (bytes_num--) {
        bytes_cnt = buf_size;
        buf_ptr = buf;
        byte_prev = byte_fill;
        while (bytes_cnt) {
            byte_tmp = *buf_ptr;
            *buf_ptr = byte_prev;
            byte_prev = byte_tmp;
            buf_ptr++;
            bytes_cnt--;
        }
    }
}

void buf_slice_bits(U8 *buf_in, U32 buf_in_size, U8 *buf_out, U32 buf_out_size, U8 bits_num) {
    U8 bits_mask;
    if (bits_num) {
        if (bits_num > 8) bits_num = 8; //limit bits number
        bits_mask = bits_rfill_U8(bits_num); //create mask for bits
        while (buf_out_size--) {
            *buf_out = (*buf_in) & bits_mask; //copy next byte
            buf_shr_bits(buf_in, buf_in_size, bits_num, 0); //shift input buffer
            buf_out++; //goto next byte of output buffer
        }
    }
}

void buf_repl_bytes_byindex(U8 *buf_in, U32 buf_in_size, U8 *buf_out, U8 *buf_repl, U8 bytes_mask) {
    while (buf_in_size--) {
        *buf_out = *(buf_repl + ((*buf_in) & bytes_mask));
        buf_in++;
        buf_out++;
    }
}

S32 ihex2buf(U8 *astr, U8 *buf, U32 buf_size) {
    S32 S32_01;
    S32_01 = -1; //index of last written byte
    if (strlen((const char *)astr) >= 11) {
        U8 U8_01;
        //check record type
        if ((astr[0] == ':') && (asc2bin_U32(&astr[7], 2) == 0)) {
            U8_01 = asc2bin_U32(&astr[1], 2); //number of bytes in string
            S32_01 = asc2bin_U32(&astr[3], 4); //address of first byte
            astr += 9; //pointer to first data byte
            while (U8_01) { //byte read loop
                //read next byte
                if (S32_01 < buf_size) buf[S32_01] = asc2bin_U32(astr, 2);
                U8_01--;
                S32_01++;
                astr += 2;
            }
        }
    }
    return(S32_01); /*return index of maximum written byte*/
}


//time operations

U16 minsec_pack(U8 minutes,U8 seconds) {
    return(((U16)minutes * 100) + seconds);
}

U8 minsec_secs(U16 time) {
    return(time % 100);
}

U8 minsec_mins(U16 time) {
    return(time / 100);
}

U16 minsec_incsecs(U16 time) {
    time++;
    if (minsec_secs(time) == 60) time += 40;
    return(time);
}

U16 minsec_decsecs(U16 time) {
    time--;
    if (minsec_secs(time) == 99) time -= 40;
    return(time);
}

U16 minsec_incmins(U16 time) {
    if (time < 65300) time += 100;
    return(time);
}

U16 minsec_decmins(U16 time) {
    if (time >= 100) time -= 100;
    return(time);
}


//if (year mod 4 != 0) {use 28 for days in February}
//  else if  (year mod 400 == 0) {use 29 for days in February}
//      else if (year mod 100 == 0) {use 28 for days in February}
//          else {use 29 for days in February}
U8 date_yearleap(U16 year) {
    return(((U8)year & 3) == 0);
}

#if (USE_GCCAVR == 1)
 U8 PROGMEM date_whatday_table[] = {0, 0, 3, 3, 6, 1, 4, 6, 2, 5, 0, 3, 5};
#else
 U8 date_whatday_table[]         = {0, 0, 3, 3, 6, 1, 4, 6, 2, 5, 0, 3, 5};
#endif

U8 date_whatday(U8 date, U8 month, U16 year) {
    U16 tempyear;
    U8 accvalue;
    tempyear = 1990;
    accvalue=0;
    while (tempyear != year) {
        accvalue++;
        if (date_yearleap(tempyear)) accvalue++;
        tempyear++;
    }
    if ((month > 2) && (date_yearleap(tempyear))) accvalue++;
    #if (defined(USE_GCCAVR))
        accvalue += pgm_read_byte(&date_whatday_table[month]);
    #else
        accvalue += date_whatday_table[month];
    #endif
    accvalue += date;
    return(accvalue % 7);
}

/* psychrometer method humidity calculation,temperatures in 0.1 degree format */
#if (USE_GCCAVR == 1)
    U16 PROGMEM hum_psych_calc_table[] = {56,65,72,78,86,93,101,109,118,125,134,144,153,162,175,180,191,200,212,222,233,245,255,266,275,285,299,304,318,330,337,346,360,369,376,381,387,393,395,402,400,412,409,421,425,429,441,437,440,444,443,441,453,463,477,490,505,513,521,536,537};
#else
    U16 hum_psych_calc_table[] = {56,65,72,78,86,93,101,109,118,125,134,144,153,162,175,180,191,200,212,222,233,245,255,266,275,285,299,304,318,330,337,346,360,369,376,381,387,393,395,402,400,412,409,421,425,429,441,437,440,444,443,441,453,463,477,490,505,513,521,536,537};
#endif

/* humidity calculation table for 0-50 deg 2 deg step */
U16 hum_psych_calc(S16 trm_dry, S16 trm_wet) {
    U16 result = 0;
    // check for temperature errors
    if ((trm_err(trm_dry)) || (trm_err(trm_wet)) || (trm_dry < trm_wet)) result = (TRM_ER | TRM_ER_OTHER);
    else {
        if ((trm_dry <= trm_wet) || (trm_dry > 1500)) result = 999;
        else {
            U8 U8_01;
            result = trm_dry / 10;
            U8_01=result >> 1; //temperature index in table
            result *= 10;
            #if (USE_GCCAVR == 1)
                result=((((unsigned long)pgm_read_word(&hum_psych_calc_table[U8_01]))*(20ul+result-trm_dry)) +
                       (((unsigned long)pgm_read_word(&hum_psych_calc_table[U8_01+1]))*(trm_dry-result)))/20ul;
            #else
                result=((((unsigned long)hum_psych_calc_table[U8_01])*(20ul+result-trm_dry)) +
                       (((unsigned long)hum_psych_calc_table[U8_01+1])*(trm_dry-result)))/20ul;
            #endif
            if ((trm_dry - trm_wet) > result) result = 0x8000;
            else result = 1000 - ((1000ul * (trm_dry - trm_wet)) / result);
        }
    }
    if (result >= 1000) result = 999;
    return(result);
}

