#if !defined(MATH_INCLUDED)
    #define MATH_INCLUDED

#include "ul_misc.h"


// returns offset of structure member
#if (!defined(offsetof))
    #define offsetof(s,m) ((size_t)&(((s*)0)->m))
#endif



//multiplication/division by constants-----------------------------------------------------------------------------------------------------------

// unsigned 8-bit multiplication by 5 (using shifts)
U8 mul_by5_U8_soft(U8 data_in);
// unsigned 16-bit multiplication by 5 (using shifts)
U8 mul_by5_U16_soft(U16 data_in);

// unsigned 8-bit division by 3
U8 div_by3_U8_soft(U8 data_in);
// unsigned 8-bit division by 3 (using multiply)
U8 div_by3_U8_hmul(U8 data_in);
// unsigned 16-bit division by 3 (using shifts)
U16 div_by3_U16_soft(U16 data_in);
// unsigned 16-bit division by 3 (using multiply)
U16 div_by3_U16_hmul(U16 data_in);
// unsigned 8-bit division by 5 (using shifts)
U8 div_by5_U8_soft(U8 data_in);
// unsigned 8-bit division by 5 (using multiply)
U8 div_by5_U8_hmul(U8 data_in);
// unsigned 16-bit division by 5 (using shifts)
U16 div_by5_U16_soft(U16 data_in);
// unsigned 16-bit division by 5 (using multiply)
U16 div_by5_U16_hmul(U16 data_in);

// counts number of '1' bits in 8-bit
U8 onebits_cnt_U8(U8 data);



//Parity-----------------------------------------------------------------------------------------------------------

// macros calculates parity of 8-bit data
#define inc_parity_U8(data) { \
    data = data ^ (data >> 1); \
    data = data ^ (data >> 2); \
    data = data ^ (data >> 4); \
    data &= 1; \
 }
// function calculates parity of 8-bit data
U8 parity_U8(U8 data);



//BCD conversions-----------------------------------------------------------------------------------------------------------

// 8-bit BCD->BIN conversion
U8 bcd2bin_U8( U8 bcd );
// 8-bit BIN->BCD conversion
U8 bin2bcd_U8( U8 bin );

// 32-bit non-optimized BIN->BCD conversion
U32 bin2bcd_U32_soft(U32 data, U8 result_bytes);
// 32-bit non-optimized BCD->BIN conversion
U32 bcd2bin_U32_soft(U32 data, U8 input_bytes);

// AVR-optimized conversions
#if (USE_GCCAVR == 1)
    U32 bin2bcd_U32_avr(U32 data, U8 result_bytes);
    U32 bcd2bin_U32_avr(U32 data, U8 input_bytes);
#endif


// ASCII-----------------------------------------------------------------------------------------------------------

extern U8 asc_point;
extern U8 asc_space;
extern U8 asc_minus;

// ASCII HEX-symbol to binary conversion, for non-numeric symbol returns 0xF0
U8 asc2bin_U8(U8 data);
// convert low nibble to ASCII HEX-symbol ('0'-'9','A'-'F')
U8 bin2asc_U8(U8 data);

// load size ASCII HEX digits from *buf (MSB first) to uint32 (sign of size defines direction of buffer scanning)
U32 asc2bin_U32(U8 *buf, S8 size);
// convert unsigned 32-bit data to ASCII symbols array (buf* - pointer to LSB, size - number of symbols, size < 0 - decrement pointer, size > 0 - increment pointer)
void bin2asc_U32(U32 data, U8 *buf, S8 size);

// store fixed-point signed 32-bit data to ASCII symbols array ( buf - pointer to LSB ( buf-1 - next digit)
//   (options & 7) + 1 - number of digits,
//   ((options >> 4) & 7) - point position (counting from right, 0 - don't show point) - asc_point,
//   ((options & 0x08) == 0x08) - don't show leading zeroes (show asc_space),
//   ((options & 0x80) == 0x80) - convert data to BCD
void num2asc_S32(S32 data, U8 *buf, U8 options);



// Gray code-----------------------------------------------------------------------------------------------------------

// BIN to Gray 32-bit conversion
U32 bin2gray_U32(U32 data);
// BIN to Gray 8-bit conversion
U8 bin2gray_U8(U8 data);
// Gray to BIN 8-bit conversion
U8 gray2bin_U8(U8 data);



// CRC-----------------------------------------------------------------------------------------------------------

// CRC8 Dallas (021cb801000000a2=0x00)
#define CRC8_DS_INIT 0x00
#define inc_crc8_ds_upd(data, crc_prev) { \
    data ^= crc_prev; \
    crc_prev = 0; \
    if(data & 0x01) crc_prev = 0x5e; \
    if(data & 0x02) crc_prev ^= 0xbc; \
    if(data & 0x04) crc_prev ^= 0x61; \
    if(data & 0x08) crc_prev ^= 0xc2; \
    if(data & 0x10) crc_prev ^= 0x9d; \
    if(data & 0x20) crc_prev ^= 0x23; \
    if(data & 0x40) crc_prev ^= 0x46; \
    if(data & 0x80) crc_prev ^= 0x8c; \
}
U8 crc8_ds_upd(U8 data, U8 crc_prev);
U16 crc8_ds_buf(U8 *buf, U16 buf_size, U8 crc_prev);

// CRC16 Dallas (data=0x75, crc_in=0x90f1, crc_out=0x6390)
#define CRC16_DS_INIT 0x0000
#define CRC16_DS_POLY 0x4002
#define inc_crc16_ds_upd(data, crc_prev) { \
    U8 cnt_bits, flag_xor; \
    for (cnt_bits = 8; cnt_bits; cnt_bits--) { \
        flag_xor = (data ^ (U8) crc_prev) & 1; \
        if (flag_xor) { \
            crc_prev ^= CRC16_DS_POLY; \
            crc_prev >>= 1; \
            crc_prev |= 0x8000; \
        } else crc_prev >>= 1; \
        data >>= 1; \
    } \
}
U16 crc16_ds_upd(U8 data, U16 crc_prev);
U16 crc16_ds_buf(U8 *buf, U16 buf_size, U16 crc_prev);


// CRC16 Modbus (0401000a000d=0x98dd; 0401020a11=0x50b3; 0402000a000d=0x9899;)
#define CRC16_MODBUS_INIT 0xffff
#define CRC16_MODBUS_POLY 0xa001
#define inc_crc16_modbus_upd(data, crc_prev) { \
    U8 cnt_bits, flag_xor; \
    crc_prev ^= data; \
    for (cnt_bits = 8; cnt_bits; cnt_bits--) { \
        flag_xor = crc_prev & 1; \
        crc_prev >>= 1; \
        if (flag_xor) crc_prev ^= CRC16_MODBUS_POLY; \
    } \
}
U16 crc16_modbus_upd(U8 data, U16 crc_prev);
U16 crc16_modbus_buf(U8 *buf, U16 buf_size, U16 crc_prev);


// CRC16 CCIT (''=0x1D0F; 'A'=0x9479; '123456789'=0xE5CC;)
#define CRC16_CCIT_INIT 0xffff
#define CRC16_CCIT_POLY 0x1021
#define inc_crc16_ccit_upd(data, crc_prev) { \
    U8 cnt_bits, flag_xor; \
    for (cnt_bits = 8; cnt_bits; cnt_bits--) { \
        flag_xor = (crc_prev >> 8) & 0x80; \
        crc_prev <<= 1; \
        if (data & 0x80) crc_prev |= 1; \
        if (flag_xor) crc_prev ^= CRC16_CCIT_POLY; \
        data <<= 1; \
    } \
}
U16 crc16_ccit_upd(U8 data, U16 crc_prev);
#define inc_crc16_ccit_augment(crc_prev) { \
    U8 cnt_bits, flag_xor; \
    for (cnt_bits = 16; cnt_bits; cnt_bits--) { \
        flag_xor = (crc_prev >> 8) & 0x80; \
        crc_prev <<= 1; \
        if (flag_xor) crc_prev ^= CRC16_CCIT_POLY; \
    } \
}
U16 crc16_ccit_augment(U16 crc_prev);
U16 crc16_ccit_buf(U8 *buf, U16 buf_size, U16 crc_prev);


// CRC7 MMC (4000000000=4a)
#define CRC7_MMC_INIT  0
#define CRC7_MMC_POLY  0x09
#define inc_crc7_mmc_upd(data, crc_prev) { \
    U8 cnt_bits; \
    for (cnt_bits = 8; cnt_bits; cnt_bits--) { \
        crc_prev <<= 1; \
        if (((crc_prev & 0x80)) ^ ((data & 0x80))) crc_prev ^= CRC7_MMC_POLY; \
        data <<= 1; \
    } \
}
U8 crc7_mmc_upd(U8 data, U8 crc_prev);
U16 crc7_mmc_buf(U8 *buf, U16 buf_size, U8 crc_prev);
  

// CRC16 MMC (sort of CRC16 CCIT) ('123456789'=31c3, 'A'=58e5)
#define CRC16_MMC_INIT 0
#define CRC16_MMC_POLY 0x1021
#define inc_crc16_mmc_upd(data, crc_prev) { \
    U8 cnt_bits, flag_xor; \
    for (cnt_bits = 8; cnt_bits; cnt_bits--) { \
        flag_xor = ((crc_prev >> 8) & 0x80) ^ (data & 0x80); \
        data <<= 1; \
        crc_prev <<= 1; \
        if (flag_xor) crc_prev ^= CRC16_MMC_POLY; \
    } \
 }
U16 crc16_mmc_upd(U8 data, U16 crc_prev);
U16 crc16_mmc_buf(U8 *buf, U16 buf_size, U16 crc_prev);



// Hashes --------------------------------------------------------------------------------------------------

#define HASH_RS_INIT 0
U32 hash_rs( U8 *buf, U32 buf_size );

#define HASH_LY_INIT 0
U32 hash_ly( U8 *buf, U32 buf_size );



// Temperature processing -----------------------------------------------------------------------------------
#define TRM_ER       0x8000
#define TRM_ER_OTHER 0x01 // temperature unknown error value
#define TRM_ER_OVER  0x02 // temperature overheat (or floating sensor) value
#define TRM_ER_UNDER 0x03 // temperature underheat (or shorted sensor) value

// returns temperature error code for trm value
U8 trm_err(S16 trm);



// bit/byte buffer operations --------------------------------------------------------------------------------

// set bit bit_num in bit buffer buf
#define buf_bit_set(buf, bit_num) (*((U8 *)buf[bit_num >> 3]) |= (1 << (bit_num & 7)))
// clear bit bit_num in bit buffer buf
#define buf_bit_clr(buf, bit_num) (*((U8 *)buf[bit_num >> 3]) &= ~(1 << (bit_num & 7)))
// read bit bit_num from bit buffer buf
#define buf_bit_rd(buf, bit_num) (*((U8 *)buf[bit_num >> 3]) & (1 << (bit_num & 7)))

// returns byte filled with bits_num '1' bits from right
U8 bits_rfill_U8(U8 bits_num);
// returns byte filled with bits_num '1' bits from left
U8 bits_lfill_U8(U8 bits_num);

// mirror bits of byte (inline)
#define inc_bits_mirror_U8(data, result) { \
    result = 0; \
    if (data & 0x01) result  = 0x80; \
    if (data & 0x02) result |= 0x40; \
    if (data & 0x04) result |= 0x20; \
    if (data & 0x08) result |= 0x10; \
    if (data & 0x10) result |= 0x08; \
    if (data & 0x20) result |= 0x04; \
    if (data & 0x40) result |= 0x02; \
    if (data & 0x80) result |= 0x01; \
    return(result); \
}
// returns mirrored byte
U8 bits_mirror_U8(U8 data);

// mirror 32-bit word (inline)
#define inc_bits_mirror_U32(data) { \
    data = ((data & 0x55555555) <<  1) | ((data & 0xAAAAAAAA) >>  1); \
    data = ((data & 0x33333333) <<  2) | ((data & 0xCCCCCCCC) >>  2); \
    data = ((data & 0x0F0F0F0F) <<  4) | ((data & 0xF0F0F0F0) >>  4); \
    data = ((data & 0x00FF00FF) <<  8) | ((data & 0xFF00FF00) >>  8); \
    data = ((data & 0x0000FFFF) << 16) | ((data & 0xFFFF0000) >> 16); \
}
// mirror bits of 32-bit word
U32 bits_mirror_U32(U32 data);

// count number of leading zero bits in a byte
U8 bits_nlz_U8(U8 data);

// returns log2 of 16-bit number (if not power of 2, returns 0xff)
U8 log2ex_U16(U16 data);

// counts number of bytes abyte in buffer buf
U32 buf_cnt_bytes(U8 *buf, U32 buf_size, U8 abyte);
// mirrors bytes in buffer
void buf_mirror_bytes(U8 *buf, U32 buf_size);
// mirrors byte bits of bits in buffer
void buf_mirror_bytes_bits(U8 *buf, U32 buf_size);
// mirrors bits in buffer
void buf_mirror_bits(U8 *buf, U32 buf_size);

// right shift of buffer for 1 bit
void buf_shr_bits(U8 *buf, U32 buf_size, U32 bits_num, U8 bit_fill);
// left shift of buffer for 1 bit
void buf_shl_bits(U8 *buf, U32 buf_size, U32 bits_num, U8 bit_fill);
// right shift of buffer for 1 byte
void buf_shr_bytes(U8 *buf, U32 buf_size, U32 bytes_num, U8 byte_fill);
// left shift of buffer for 1 byte
void buf_shl_bytes(U8 *buf, U32 buf_size, U32 bytes_num, U8 byte_fill);

// slice input buffer *buf_in (destroyed!!) of size buf_in_size by bits_num slices to *buf_out
void buf_slice_bits(U8 *buf_in, U32 buf_in_size, U8 *buf_out, U32 buf_out_size, U8 bits_num);
// replace bytes in buffer *buf_in, to buffer *buf_out, anded with mask byte_mask, by bytes from *buf_repl , indexed by buf
void buf_repl_bytes_byindex(U8 *buf_in, U32 buf_in_size, U8 *buf_out, U8 *buf_repl, U8 bytes_mask);



// HEX ----------------------------------------------------------------------------------------------------------------

// loads contents of Intel HEX string *astr to buffer *buf of size buf_size, returns maximum modified address
S32 ihex2buf(U8 *astr, U8 *buf, U32 buf_size);



// packed time in ((minutes * 100) + seconds) format manipulation ----------------------------------------------------

// pack minutes,seconds to number ((minutes * 100) + seconds)
U16 minsec_pack(U8 minutes,U8 seconds);
// get seconds value from packed time
U8 minsec_secs(U16 time);
// get minutes value from packed time
U8 minsec_mins(U16 time);
// increment seconds value in packed time
U16 minsec_incsecs(U16 time);
// decrement seconds value in packed time
U16 minsec_decsecs(U16 time);
// increment minutes value in packed time
U16 minsec_incmins(U16 time);
// decrement minutes value in packed time
U16 minsec_decmins(U16 time);



// Date/Time ---------------------------------------------------------------------------------------------------------
// returns TRUE if year is leap (for 1901..2099 years !!!)
U8 date_yearleap(U16 year);
// returns day of week number for given date (0..6, 0 is sunday)
U8 date_whatday(U8 date, U8 month, U16 year);



// Humidity ----------------------------------------------------------------------------------------------------------
// calculate humidity from dry and wet temperature (0.1 fixed-point format)
U16 hum_psych_calc(S16 trm_dry, S16 trm_wet);

#endif

