#include "ul_rtc.h"  /*include RTC header*/

#if (defined(USE_GCCAVR))
    #include <avr/pgmspace.h>     /*Program memory usage for AVR-GCC*/
    U8 PROGMEM rtc_corr_dates_table[2][32] =
    {{29,28,27,26,31,30,29,28,26,25,31,30,28,27,26,25,30,29,28,27,25,31,30,29,27,26,25,31,29,28,27,26},
     {26,25,31,30,28,27,26,25,30,29,28,27,25,31,30,29,27,26,25,31,29,28,27,26,31,30,29,28,26,25,31,30}
    };
    struct_rtc NOINIT rtc;
#else
    U8 rtc_corr_dates_table[2][32] =
    {{29,28,27,26,31,30,29,28,26,25,31,30,28,27,26,25,30,29,28,27,25,31,30,29,27,26,25,31,29,28,27,26},
     {26,25,31,30,28,27,26,25,30,29,28,27,25,31,30,29,27,26,25,31,29,28,27,26,31,30,29,28,26,25,31,30}
    };
    struct_rtc rtc;
#endif


U8 rtc_increment(struct_rtc *rtc_data) {
    U8 U8_01, U8_02;
    U8_01 = rtc_data->month;
    U8_02 = rtc_data->date;
    // clear flags
    rtc_data->flags &= ~(FLAG_RTC_SEC_PASS|FLAG_RTC_MIN_PASS|FLAG_RTC_HR_PASS|FLAG_RTC_CORRECTED|FLAG_RTC_CORRECTED_FORW);
    rtc_data->flags |= FLAG_RTC_SEC_PASS;  // set second passed flag
    if (++(rtc_data->sec) >= 60) { // process seconds overflow
        rtc_data->sec = 0;
        rtc_data->flags |= FLAG_RTC_MIN_PASS; // set minute passed flag
        if (++(rtc_data->min) >= 60) { // process minutes overflow
            rtc_data->min = 0;
            rtc_data->flags |= FLAG_RTC_HR_PASS; // set hour passed flag
            if (++(rtc_data->hr) >= 24) {  // process hours overflow
                rtc_data->hr = 0;
                U8_02++; // calculate next date
                if (U8_02 == 32) {
                    U8_01++;
                    U8_02 = 1;
                } else if (U8_02 == 31) {
                    if ((U8_01 == 4) || (U8_01 == 6) || (U8_01 == 9) || (U8_01 == 11)) {
                        U8_01++;
                        U8_02 = 1;
                    }
                } else if (U8_02 == 30) {
                    if (U8_01==2) {U8_01++; U8_02 = 1;}
                }else if (U8_02 == 29) {
                    if ((U8_01 == 2) && ((rtc_data->year) & 3)) {
                        U8_01++;
                        U8_02 = 1;
                    }
                }
                // month overflow
                if (U8_01 >= 13) {
                    U8_01 = 1;
                    (rtc_data->year)++;
                }
            }
        }
    }
    
    rtc_data->month = U8_01;
    rtc_data->date = U8_02;

    // if needed, do time auto correction
    if (rtc_data->flags & FLAG_RTC_CORRECTION) {
        // every last sunday of march (hour++) and october (hour--) at 4 a.m.
        if (((U8_01 == 10) || (U8_01 == 3)) && (U8_02 > 24) &&
            (rtc_data->hr == 4) && (rtc_data->min == 0) && 
            (rtc_data->sec == 0)) {

            // get correction date
            U8 corr_date = rtc_data->year - 2000; // table index
            #if (defined(USE_GCCAVR))
                corr_date = pgm_read_byte(&(rtc_corr_dates_table[U8_01 & 1][corr_date]));
            #else
                corr_date = rtc_corr_dates_table[U8_01 & 1][corr_date];
            #endif
            
            // if date is correct, do time correction
            if (U8_02 == corr_date) {
                rtc_data->flags |= FLAG_RTC_CORRECTED; // set correction flag
                if (U8_01 == 3) {
                    rtc_data->hr++;
                    rtc_data->flags |= FLAG_RTC_CORRECTED_FORW; // forward correction
                } else rtc_data->hr--;
            }
        }
    }
    return rtc_data->flags;
}

void rtc_reset(struct_rtc *rtc_data) {
    memset((void *)rtc_data, 0, sizeof(struct_rtc));
    rtc_data->date = 1;
    rtc_data->month = 1;
    rtc_data->year = 2000;
}

void rtc_correct_hour(struct_rtc *rtc_data) {
    if (rtc_data->min < 30) {
        rtc_data->sec = rtc_data->min = 0;
    } else {
        rtc_data->sec = rtc_data->min = 59;
    }
}

U8 rtc_correct_winter_summer(struct_rtc *rtc_data) {

    U8 result = 0;
    
    // every last sunday of march (hour++) and october (hour--) at 4 a.m.
    U8 month = rtc_data->month;
    if (
        ((month == RTC_MONTH_OCTOBER) || (month == RTC_MONTH_MARCH)) &&
        (rtc_data->date > 24) &&
        (rtc_data->hr == 4) &&
        (rtc_data->min == 0) && 
        (rtc_data->sec == 0)
    ) {

        // get correction date
        U8 corr_date = rtc_data->year - 2000; // table index
        #if (defined(USE_GCCAVR))
            corr_date = pgm_read_byte(&(rtc_corr_dates_table[month & 1][corr_date]));
        #else
            corr_date = rtc_corr_dates_table[month & 1][corr_date];
        #endif
        if (rtc_data->date == corr_date) {
            if (month == 3) { // forward correction
                rtc_data->hr++;
                rtc_data->flags |= FLAG_RTC_CORRECTED_FORW; // forward correction
                result = 1;
            } else { // backward correction
                if (!(rtc_data->flags & FLAG_RTC_CORRECTED) ||
                    ((rtc_data->flags & FLAG_RTC_CORRECTED)&&(rtc_data->flags & FLAG_RTC_CORRECTED_FORW))
                ) {
                    rtc_data->hr--;
                    rtc_data->flags &= ~FLAG_RTC_CORRECTED_FORW;
                    result = 2;
                }
            }
            rtc_data->flags |= FLAG_RTC_CORRECTED; // set correction flag
        }
    }
    
    return result;
}

