/** \addtogroup group_rtc software RTC (программные часы реального времени)
  */

#if ((!defined(UL_RTC_INCLUDED)) || (defined __DOXYGEN__))
    #define UL_RTC_INCLUDED

#include "ul_misc.h"    /*include misc definitions/functions types*/

/** \brief RTC structure<br />структура часов \ingroup group_rtc */
typedef struct {
    U8 sec;   /**< \brief seconds (0..59)  <br/>секунды (0..59)       \ingroup group_rtc */
    U8 min;   /**< \brief minutes (0..59)  <br/>минуты (0..59)        \ingroup group_rtc */
    U8 hr;    /**< \brief hours (0..23)    <br/>часы (0..23)          \ingroup group_rtc */
    U8 date;  /**< \brief date (1..31)     <br/>дата (1..31)          \ingroup group_rtc */
    U8 month; /**< \brief month (1..12)    <br/>месяц (1..12)         \ingroup group_rtc */
    U16 year; /**< \brief year (1900..2100)<br/>год (1900..2100)      \ingroup group_rtc */
    U8 flags; /**< \brief RTC flags        <br/>флаги RTC             \ingroup group_rtc */
    U16 crc;  /**< \brief crc of structure <br/>контр.сумма структуры \ingroup group_rtc */
} struct_rtc;

struct_rtc rtc; /**< \brief RTC structure \ingroup group_rtc */

#define RTC_MONTH_JANUARY   1
#define RTC_MONTH_FEBRUARY  2
#define RTC_MONTH_MARCH     3
#define RTC_MONTH_APRIL     4
#define RTC_MONTH_MAY       5
#define RTC_MONTH_JUNE      6
#define RTC_MONTH_JULY      7
#define RTC_MONTH_AUGUST    8
#define RTC_MONTH_SEPTEMBER 9
#define RTC_MONTH_OCTOBER   10
#define RTC_MONTH_NOVEMBER  11
#define RTC_MONTH_DECEMBER  12

#define FLAG_RTC_SEC_PASS       0x01 /**< \brief =1 every second passed<br/>=1 каждую секунду \ingroup group_rtc */
#define FLAG_RTC_MIN_PASS       0x02 /**< \brief =1 every minute passed<br/>=1 каждую минуту \ingroup group_rtc */
#define FLAG_RTC_HR_PASS        0x04 /**< \brief =1 every hour passed<br/>=1 каждый час \ingroup group_rtc */
#define FLAG_RTC_CORRECTED      0x08 /**< \brief =1 if was time correction<br/>=1 если была коррекция на летнее/зимнее время \ingroup group_rtc */
#define FLAG_RTC_CORRECTED_FORW 0x10 /**< \brief =1 if was hour-forward time correction (otherwise hour-backward)<br/>=1 если была коррекция на час вперед (иначе на час назад) \ingroup group_rtc */
#define FLAG_RTC_CORRECTION     0x20 /**< \brief =1 for auto-correction<br/>=1 для автоматического перехода на зимнее/летнее время \ingroup group_rtc */
#define FLAG_RTC_SLEEP          0x80 /**< \brief =1 if system in sleep mode<br/>=1 в режиме сна \ingroup group_rtc */

U8 rtc_corr_dates_table[2][32];

/** \brief increment time of RTC structure by 1 second<br />увеличить время в структуре часов на 1 секунду \ingroup group_rtc */
U8 rtc_increment(struct_rtc *rtc_data);
/** \brief reset RTC structure<br />обнулить структуру часов \ingroup group_rtc */
void rtc_reset(struct_rtc *rtc_data);

// correct RTC to closest hour
void rtc_correct_hour(struct_rtc *rtc_data);
// correct RTC to winter/summer time. Returns 0 for no-correction, 1 for forward correction, 2 for backward correction
U8 rtc_correct_winter_summer(struct_rtc *rtc_data);

#endif

