/** \addtogroup group_timing timing (таймеры) */

#if ((!defined(TIMING_INCLUDED)) || (defined __DOXYGEN__))
    #define TIMING_INCLUDED

#include "ul_misc.h"

// timing flags
extern volatile U8 flags_tmr;
#define FLAG_TMR_1    0x01
#define FLAG_TMR_1S   0x02
#define FLAG_TMR_01   0x04
#define FLAG_TMR_01S  0x08
#define FLAG_TMR_001  0x10
#define FLAG_TMR_001S 0x20
#define FLAG_TMR_STRT 0x40
#define FLAG_TMR_WAIT 0x80


#define TIME_001     ((F_TMR + 50) / 100ul)
#define TIME_01      ((F_TMR + 5) / 10ul)
#define TIME_1       (F_TMR)
extern U8 tmr_001;
extern U16 tmr_01;
extern U16 tmr_1;
extern U8 tmr_strt;


#define ftmr_1()    (flags_tmr & FLAG_TMR_1S)
#define ftmr_01()   (flags_tmr & FLAG_TMR_01S)
#define ftmr_001()  (flags_tmr & FLAG_TMR_001S)
#define ftmr_strt() (flags_tmr & FLAG_TMR_STRT)


void delay_short(U16 adelay);
void delay_tmr_ticks(U16 adelay);


// timers update (insert in main timer ISR)
#define tmr_flags_upd() { \
    U8 flags_tmp = flags_tmr & ~FLAG_TMR_WAIT; \
    tmr_001++; \
    if (tmr_001 >= TIME_001) { \
        tmr_001 = 0; \
        flags_tmp |= FLAG_TMR_001; \
    } \
    tmr_01++; \
    if (tmr_01 >= TIME_01) { \
        tmr_01 = 0; \
        flags_tmp |= FLAG_TMR_01; \
        if (tmr_strt < TIME_STARTUP) tmr_strt++; \
        else flags_tmp |= FLAG_TMR_STRT; \
    } \
    tmr_1++; \
    if (tmr_1 >= TIME_1) { \
        tmr_1 = 0; \
        flags_tmp |= FLAG_TMR_1; \
    } \
    flags_tmr = flags_tmp; \
}


// timers synchronize (insert at the start of main cycle)
#define tmr_flags_sync() { \
    cli(); \
    U8 flags_tmp = flags_tmr & ~(FLAG_TMR_001S | FLAG_TMR_01S | FLAG_TMR_1S); \
    if (flags_tmp & FLAG_TMR_001) { \
        flags_tmp = (flags_tmp & ~FLAG_TMR_001) | FLAG_TMR_001S; \
        if (flags_tmp & FLAG_TMR_01) { \
            flags_tmp = (flags_tmp | FLAG_TMR_01S) & ~FLAG_TMR_01; \
            if (flags_tmp & FLAG_TMR_1) { \
                flags_tmp = (flags_tmp | FLAG_TMR_1S) & ~FLAG_TMR_1; \
            } \
        } \
    } \
    flags_tmr = flags_tmp; \
    sei(); \
}

#endif

