#if (!defined(MAIN_INCLUDED)) // prevent recursive includes
    #define MAIN_INCLUDED 1


// Configuration flags
#define USE_GCCAVR       1 // use AVR-GCC specific library code


#define bin2bcd_U32       bin2bcd_U32_avr // use AVR-optimized BIN->BCD conversion
#define bcd2bin_U32       bcd2bin_U32_avr // use AVR-optimized BCD->BIN conversion
#define delay_us          _delay_us
#define delay_ms          _delay_ms
#define wdr()             wdr_avr()


// Debug flags
#define DEBUG_IND         0 // debug indication
#define TEST_IND          0 // test indication on start
#define DEBUG_KBD         0 // debug keyboard
#define DEBUG_REMOTE      0
#define DEBUG_TEMPERATURE 0



#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include <util/delay.h>

#include "ul_misc.h"
#include "ul_math.h"


// timing, RTC
#define  TMR0_FRQ               10000UL          // timer 0 overflow frequency
#define  TMR0_PRE               8UL              // timer 0 prescaler
#define  TCNT0_VAL              (256UL - (F_CPU / TMR0_PRE / TMR0_FRQ))
#define  TIME_STARTUP           2                // startup delay (in 100ms increments)

#define  F_TMR                  TMR0_FRQ         // main timer frequency
#include "ul_timing.h"
#include "ul_rtc.h"


// i2c
#define  UL_USE_I2C_AVR         1                // use hardware TWI
#define  I2C_BAUD               200000UL         // SCL frequency
#include "ul_i2c.h"
#include "ul_rtc_ds3231.h"

// 1-wire
#define  DS1W_OUT_PORT          PORTB            // output port
#define  DS1W_DIR_PORT          DDRB             // direction port
#define  DS1W_IN_PORT           PINB             // input port
#define  DS1W_BIT               (1 << PB0)       // port bit mask
#define  DS1W_DELAY_RESET_TICKS 8                // reset delay timer ticks
#include "ul_1wire.h"

#define  F_REMOTE               F_TMR            // remote receiver poll frequency
#define  REMOTE_TYPE            REMOTE_TYPE_SONY // remote protocol type
#define  REMOTE_BITS_MAX        12               // received data bits
#define  REMOTE_NOPRESS_MIN     30               // minimum no-press delay
#define  REMOTE_TOL_MINUS       2                // remote capture minus-tolerance
#define  REMOTE_TOL_PLUS        2                // remote capture plus-tolerance
#define  read_remote()          (PINB & (1 << PB2)) // read remote receiver pin state
#include "ul_remote.h"

#include "setup.h"
#include "cfg.h"
#include "ind.h"
#include "kbd.h"
#include "rtc.h"
#include "temperature.h"
#include "lighting.h"

int main(void);

#endif

